@@ -94,6 +94,11 @@ ip_pat = re.compile(r"""\b(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25
 dev_pat = re.compile(r"""/dev/.+""", re.IGNORECASE)
 usb_pat = re.compile(r"""(\d+):(\d+)""", re.IGNORECASE)
 
+### **********Lambda Function UniStar for checking type of arguments to constructor of class event*******************************
+
+UniStr = lambda title: isinstance(title, str) and utils.xrstrip(title, '\x00')[:128] or utils.xrstrip(title, '\x00')[:128].encode('utf-8')
+
+
 #
 # Event Wrapper Class for pipe IPC
 #
@@ -102,13 +107,13 @@ class Event(object):
     def __init__(self, device_uri, printer_name, event_code,
                  username=prop.username, job_id=0, title='',
                  timedate=0):
-
-        self.device_uri = unicode(utils.xrstrip(device_uri, '\x00'))[:128].encode('utf-8')
-        self.printer_name = unicode(utils.xrstrip(printer_name, '\x00'))[:128].encode('utf-8')
+       # UniStr = lambda title: isinstance(title, str) and utils.xrstrip(title, '\x00')[:128] or utils.xrstrip(title, '\x00')[:128].encode('utf-8')
+        self.device_uri = UniStr(device_uri)
+        self.printer_name = UniStr(printer_name)
         self.event_code = int(event_code)
-        self.username = unicode(utils.xrstrip(username, '\x00'))[:32].encode('utf-8')
+        self.username = UniStr(username)
         self.job_id = int(job_id)
-        self.title = unicode(utils.xrstrip(title, '\x00'))[:128].encode('utf-8')
+        self.title = UniStr(title)
 
         if timedate:
             self.timedate = float(timedate)
@@ -1299,6 +1304,12 @@ class Device(object):
     def openEWS_LEDM(self):
         return self.__openChannel(hpmudext.HPMUD_S_EWS_LEDM_CHANNEL)
 
+    def openLEDM(self):
+        return self.__openChannel(hpmudext.HPMUD_S_LEDM_SCAN)
+
+    def openMarvell_EWS(self):
+        return self.__openChannel(hpmudext.HPMUD_S_MARVELL_EWS_CHANNEL)
+
     def closePrint(self):
         return self.__closeChannel(hpmudext.HPMUD_S_PRINT_CHANNEL)
 
@@ -1323,6 +1334,12 @@ class Device(object):
     def closeEWS_LEDM(self):
         return self.__closeChannel(hpmudext.HPMUD_S_EWS_LEDM_CHANNEL)
 
+    def closeLEDM(self):
+        return self.__closeChannel(hpmudext.HPMUD_S_LEDM_SCAN)
+
+    def closeMarvell_EWS(self):
+        return self.__closeChannel(hpmudext.HPMUD_S_MARVELL_EWS_CHANNEL)
+
     def openCfgUpload(self):
         return self.__openChannel(hpmudext.HPMUD_S_CONFIG_UPLOAD_CHANNEL)
 
@@ -1703,7 +1720,11 @@ class Device(object):
 
             elif status_type == STATUS_TYPE_LEDM:
                 log.debug("Type 10: LEDM")
-                status_block = status.StatusType10(self)
+                status_block = status.StatusType10(self.getEWSUrl_LEDM)
+
+            elif status_type == STATUS_TYPE_LEDM_FF_CC_0:
+                log.debug("Type 11: LEDM_FF_CC_0")
+                status_block = status.StatusType10(self.getUrl_LEDM)
 
             else:
                 log.error("Unimplemented status type: %d" % status_type)
@@ -1805,27 +1826,44 @@ class Device(object):
                                     'rr' : rr,
                                   })
 
+                #Check if device itself is sending the supplies info. If so, then in that case we need not check model.dat static data and
+                #compare with region, kind and type values.
+                dynamic_sku_data = False
+                for agent in agents:
+                    try:
+                        if agent['agent-sku'] != '':
+                            dynamic_sku_data = True
+                            break
+                    except:
+                        pass                                 
+
                 a, aa = 1, 1
                 while True:
-                    mq_agent_kind = self.mq.get('r%d-agent%d-kind' % (r_value, a), -1)
-
-                    if mq_agent_kind == -1:
-                        break
-
-                    mq_agent_type = self.mq.get('r%d-agent%d-type' % (r_value, a), 0)
-                    mq_agent_sku = self.mq.get('r%d-agent%d-sku' % (r_value, a), '')
-
-                    found = False
-
-                    log.debug("Looking for kind=%d, type=%d..." % (mq_agent_kind, mq_agent_type))
-                    for agent in agents:
+                    if dynamic_sku_data:
+                        if a > len(agents):
+                            break 
+                        agent = agents[a-1]
+                        mq_agent_sku = agent['agent-sku']
                         agent_kind = agent['kind']
                         agent_type = agent['type']
+                        found = True
+                    else:
+                        mq_agent_kind = self.mq.get('r%d-agent%d-kind' % (r_value, a), -1)
+                        if mq_agent_kind == -1:
+                            break
+                        mq_agent_type = self.mq.get('r%d-agent%d-type' % (r_value, a), 0)
+                        mq_agent_sku = self.mq.get('r%d-agent%d-sku' % (r_value, a), '')
+                        found = False
 
-                        if agent_kind == mq_agent_kind and \
-                           agent_type == mq_agent_type:
-                           found = True
-                           break
+                        log.debug("Looking for kind=%d, type=%d..." % (mq_agent_kind, mq_agent_type))
+                        for agent in agents:
+                            agent_kind = agent['kind']
+                            agent_type = agent['type']
+
+                            if agent_kind == mq_agent_kind and \
+                               agent_type == mq_agent_type:
+                                   found = True
+                                   break
 
                     if found:
                         log.debug("found: r%d-kind%d-type%d" % (r_value, agent_kind, agent_type))
@@ -1848,7 +1886,7 @@ class Device(object):
                         # if agent health is OK, check for low supplies. If low, use
                         # the agent level trigger description for the agent description.
                         # Otherwise, report the agent health.
-                        if (status_code == STATUS_PRINTER_IDLE or status_code == STATUS_PRINTER_OUT_OF_INK) and \
+                        if (status_code == STATUS_PRINTER_POWER_SAVE or status_code == STATUS_PRINTER_IDLE or status_code == STATUS_PRINTER_OUT_OF_INK) and \
                             (agent_health == AGENT_HEALTH_OK or
                              (agent_health == AGENT_HEALTH_FAIR_MODERATE and agent_kind == AGENT_KIND_HEAD)) and \
                             agent_level_trigger >= AGENT_LEVEL_TRIGGER_MAY_BE_LOW:
@@ -2111,6 +2149,12 @@ class Device(object):
     def readEWS_LEDM(self, bytes_to_read, stream=None, timeout=prop.read_timeout, allow_short_read=True):
         return self.__readChannel(self.openEWS_LEDM, bytes_to_read, stream, timeout, allow_short_read)
 
+    def readLEDM(self, bytes_to_read, stream=None, timeout=prop.read_timeout, allow_short_read=True):
+        return self.__readChannel(self.openLEDM, bytes_to_read, stream, timeout, allow_short_read)
+
+    def readMarvell_EWS(self, bytes_to_read, stream=None, timeout=prop.read_timeout, allow_short_read=True):
+        return self.__readChannel(self.openMarvell_EWS, bytes_to_read, stream, timeout, allow_short_read)
+
     def readSoapFax(self, bytes_to_read, stream=None, timeout=prop.read_timeout, allow_short_read=True):
         return self.__readChannel(self.openSoapFax, bytes_to_read, stream, timeout, allow_short_read)
 
@@ -2194,6 +2238,12 @@ class Device(object):
     def writeEWS_LEDM(self, data):
         return self.__writeChannel(self.openEWS_LEDM, data)
 
+    def writeLEDM(self, data):
+        return self.__writeChannel(self.openLEDM, data)
+
+    def writeMarvell_EWS(self, data):
+        return self.__writeChannel(self.openMarvell_EWS, data)
+
     def writeCfgDownload(self, data):
         return self.__writeChannel(self.openCfgDownload, data)
 
@@ -2441,6 +2491,21 @@ class Device(object):
             self.closeEWS()
 
     def getEWSUrl_LEDM(self, url, stream, footer=""):
+        try:
+            url2 = "%s&loc=%s" % (self.device_uri.replace('hpfax:', 'hp:'), url)
+            data = self
+            opener = LocalOpenerEWS_LEDM({})
+            try:
+                if footer:
+                    return opener.open_hp(url2, data, footer)
+                else:
+                    return opener.open_hp(url2, data)
+            except Error:
+                log.debug("Status read failed: %s" % url2)
+        finally:
+            self.closeEWS_LEDM()
+
+    def getUrl_LEDM(self, url, stream, footer=""):
         try:
             url2 = "%s&loc=%s" % (self.device_uri.replace('hpfax:', 'hp:'), url)
             data = self
@@ -2449,21 +2514,66 @@ class Device(object):
                 if footer:
                     return opener.open_hp(url2, data, footer)
                 else:
-                    return opener.open_hp(url2, data) 
+                    return opener.open_hp(url2, data)
             except Error:
                 log.debug("Status read failed: %s" % url2)
 
         finally:
-            self.closeEWS_LEDM()
-    def readAttributeFromXml(self,uri,attribute):
+            self.closeLEDM()
+
+    def FetchLEDMUrl(self, url, footer=""):
+        data_fp = cStringIO.StringIO()
+        if footer:
+            data = self.getUrl_LEDM(url, data_fp, footer)
+        else:
+            data = self.getUrl_LEDM(url, data_fp)
+        if data:
+            data = data.split('\r\n\r\n', 1)[1]
+            if data:
+                data = status.clean(data)
+        return data
+
+#-------------------------For LEDM SOAP PROTOCOL(FAX) Devices----------------------------------------------------------------------#
+
+    def FetchEWS_LEDMUrl(self, url, footer=""):
+        data_fp = cStringIO.StringIO()
+        if footer:
+            data = self.getEWSUrl_LEDM(url, data_fp, footer)
+        else:
+            data = self.getEWSUrl_LEDM(url, data_fp)
+        if data:
+            data = data.split('\r\n\r\n', 1)[1]
+            if data:
+                data = status.clean(data)
+        return data
+
+    def readAttributeFromXml_EWS(self, uri, attribute):
         stream = cStringIO.StringIO()
-        data = status.StatusType10FetchUrl(self,uri)        
+        data = self.FetchEWS_LEDMUrl(uri)
         if not data:
             log.error("Unable To read the XML data from device")
             return ""
-        xmlDict = utils.XMLToDictParser().parseXML(data)  
-        return str(xmlDict[attribute])
-		
+        xmlDict = utils.XMLToDictParser().parseXML(data)
+        try:
+            #return str(xmlDict[attribute])
+            return xmlDict[attribute]
+        except:
+            return str("")
+
+#---------------------------------------------------------------------------------------------------#
+
+    def readAttributeFromXml(self,uri,attribute):
+        stream = cStringIO.StringIO()
+        data = self.FetchLEDMUrl(uri)
+        if not data:
+            log.error("Unable To read the XML data from device")
+            return ""
+        xmlDict = utils.XMLToDictParser().parseXML(data)
+        try:
+            return str(xmlDict[attribute])
+        except:
+            return str("")
+
     def downloadFirmware(self, usb_bus_id=None, usb_device_id=None): # Note: IDs not currently used
         ok = False
         filename = os.path.join(prop.data_dir, "firmware", self.model.lower() + '.fw.gz')
@@ -2541,7 +2651,7 @@ class LocalOpener(urllib.URLopener):
             return response.fp
 
 # URLs: hp:/usb/HP_OfficeJet_7500?serial=00XXXXXXXXXX&loc=/hp/device/info_device_status.xml
-class LocalOpener_LEDM(urllib.URLopener):
+class LocalOpenerEWS_LEDM(urllib.URLopener):
     def open_hp(self, url, dev, foot=""):
         log.debug("open_hp(%s)" % url)
 
@@ -2560,8 +2670,50 @@ class LocalOpener_LEDM(urllib.URLopener):
 
         reply = xStringIO()
 
-        while dev.readEWS_LEDM(8080, reply, timeout=3):
+        while dev.readEWS_LEDM(512, reply, timeout=3):
             pass
 
         reply.seek(0)
         return reply.getvalue()
+
+# URLs: hp:/usb/HP_OfficeJet_7500?serial=00XXXXXXXXXX&loc=/hp/device/info_device_status.xml
+class LocalOpener_LEDM(urllib.URLopener):
+    def open_hp(self, url, dev, foot=""):
+        log.debug("open_hp(%s)" % url)
+
+        match_obj = http_pat_url.search(url)
+        loc = url.split("=")[url.count("=")]
+
+        dev.openLEDM()
+        if foot:
+            if "PUT" in foot:
+                dev.writeLEDM("""%s""" % foot)
+            else:
+                dev.writeLEDM("""POST %s HTTP/1.1\r\nContent-Type:text/xml\r\nContent-Length:%s\r\nAccept-Encoding: UTF-8\r\nHost:localhost\r\nUser-Agent:hplip\r\n\r\n """ % (loc, len(foot)))
+                dev.writeLEDM("""%s""" % foot)
+        else:
+            dev.writeLEDM("""GET %s HTTP/1.1\r\nAccept: text/plain\r\nHost:localhost\r\nUser-Agent:hplip\r\n\r\n""" % loc)
+
+        reply = xStringIO()
+ 
+        while dev.readLEDM(512, reply, timeout=3):
+            pass
+
+        #TODO:Need to add following code in order to improve the delay.
+        #exp_end_of_data="0\r\n\r\n"
+        #num_of_bytes_read = dev.readEWS_LEDM(512, reply, timeout=5)
+
+        #while num_of_bytes_read:
+            #temp_buf = xStringIO()
+            #num_of_bytes_read = dev.readEWS_LEDM(512, temp_buf, timeout=5)
+            #reply.write(temp_buf.getvalue())
+
+            #if num_of_bytes_read == 5 and exp_end_of_data == temp_buf.getvalue():
+            #    break
+            #pass
+
+        reply.seek(0)
+        return reply.getvalue()
+
+
+
