@@ -102,6 +102,13 @@ TECH_CLASSES = [
     "Python10",
     "Mimas",
     "StingrayOJ",
+    "Copperhead",
+    "Copperhead12",
+    "Ampere",
+    "Python11",
+    "Saipan",
+    "PyramidPlus",
+    "Hbpl1",
 ]
 
 TECH_CLASSES.sort()
@@ -155,6 +162,10 @@ TECH_CLASS_PDLS = {
     "Python10"      : 'pcl3',
     "Mimas"      : 'pcl3',
     "StingrayOJ"   : 'pcl3',
+    "Copperhead"   : 'pcl3',
+    "Copperhead12"   : 'pcl3',
+    "Ampere"	   : 'pcl3',
+    "Hbpl1"         : 'hbpl1'
 }
 
 PDL_TYPE_PCL = 0  # less preferred
@@ -178,6 +189,7 @@ PDL_TYPES = { # Used to prioritize PPD file selection in prnt.cups.getPPDFile2()
     'jpg' : PDL_TYPE_HOST,
     'jetready' : PDL_TYPE_HOST,
     'jr' : PDL_TYPE_HOST,
+    'hbpl1' : PDL_TYPE_HOST,
 }
 
 
@@ -208,6 +220,7 @@ TECH_SUBCLASSES = [
     "Trim",
     "4800x1200dpi",
     "Advanced",
+    "AutoDuplex"
 ]
 
 TECH_SUBCLASSES.sort()
@@ -300,7 +313,6 @@ class ModelData:
             'power-settings': TYPE_INT,
             'pq-diag-type' : TYPE_INT,
             'r-type' : TYPE_INT,
-            'scan-style' : TYPE_INT,
             'scan-type' : TYPE_INT,
             'scan-src' : TYPE_INT,
             #'scan-duplex' : TYPE_BOOL,
