@@ -16,7 +16,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Author: Stan Dolson
+# Author: Stan Dolson , Goutam Kodu
 #
 
 # Std Lib
@@ -353,6 +353,7 @@ def copyPluginFiles(src_dir):
 
     for PRODUCT in products:
         MODEL = PRODUCT.replace('hp-', '').replace('hp_', '')
+        UDEV_SYSFS_RULES=sys_conf.get('configure','udev_sysfs_rules','no')
         for s in plugin_spec.get("products", PRODUCT).split(','):
 
             if not plugin_spec.has_section(s):
@@ -363,6 +364,13 @@ def copyPluginFiles(src_dir):
             trg = plugin_spec.get(s, 'trg', '')
             link = plugin_spec.get(s, 'link', '')
 
+           # In Cent os 5.x distro's SYSFS attribute will be used. and Other distro's uses ATTR/ATTRS attribute in rules. 
+           # Following condition to check this...
+            if UDEV_SYSFS_RULES == 'no' and 'sysfs' in src:
+                continue
+            if UDEV_SYSFS_RULES == 'yes' and 'sysfs' not in src:
+                continue
+
             if not src:
                 log.error("Missing 'src=' value in section [%s]" % s)
                 return False
@@ -435,13 +443,16 @@ def copyPluginFiles(src_dir):
     sys_state.set('plugin', "installed", '1')
     log.debug("Updating hplip.conf - eula = 1")
     sys_state.set('plugin', "eula", '1')
-
+    plugin_version = sys_conf.get('hplip', 'version', '0.0.0')
+    sys_state.set('plugin','version', plugin_version)
     return True
 
 
 def run_plugin_command(required=True, plugin_reason=PLUGIN_REASON_NONE):
     su_sudo = None
     need_sudo = True
+    name = None
+    version = None
 
     if utils.to_bool(sys_conf.get('configure', 'policy-kit')):
         try:
@@ -460,7 +471,14 @@ def run_plugin_command(required=True, plugin_reason=PLUGIN_REASON_NONE):
     if need_sudo:
         su_sudo = utils.su_sudo()
     if su_sudo is "su":
-        su_sudo = 'su -c "%s"'
+        name,version,is_su = utils.os_release()
+        log.debug("name = %s version = %s is_su = %s" %(name,version,is_su))
+        if ( name == 'Fedora' and version >= '14' and is_su == True):
+           #using su opening GUI apps fail in Fedora 14. 
+           #To run GUI apps as root, you need a root login shell (su -) in Fedora 14   
+           su_sudo = 'su - -c "%s"'
+        else:
+           su_sudo = 'su -c "%s"'
         password_f = "get_password_ui"    
     if su_sudo is None:
         log.error("Unable to find a suitable sudo command to run 'hp-plugin'")
@@ -470,16 +488,25 @@ def run_plugin_command(required=True, plugin_reason=PLUGIN_REASON_NONE):
     if not required:
         req = '--optional'
 
+
     if utils.which("hp-plugin"):
-        cmd = su_sudo % ("hp-plugin -u %s --reason %s" % (req, plugin_reason))
+        p_path="hp-plugin"
     else:
-        cmd = su_sudo % ("python ./plugin.py -u %s --reason %s" % (req, plugin_reason))
+        p_path="python ./plugin.py"
+
+    if 'gksu' in su_sudo:
+        cmd = su_sudo % ("%s -u %s --reason %s" % (p_path, req, plugin_reason))
+        cmd +=" -m" 
+        cmd += (" \"hp-plugin:- HP Device requires to install HP proprietary plugin. Please enter root password to continue\"")
+    else:
+        cmd = su_sudo % ("%s -u %s --reason %s To_install_plugin_for_HP_Device" % (p_path, req, plugin_reason))
 
     log.debug("%s" % cmd)
     if password_f is not None:
         status, output = utils.run(cmd, log_output=True, password_func=password_f, timeout=1)
     else:
         status, output = utils.run(cmd, log_output=True, password_func=None, timeout=1)
+    
     return (status == 0, True)
 
 
