@@ -24,7 +24,21 @@ from __future__ import division
 # Std Lib
 import struct
 import cStringIO
-import xml.parsers.expat as expat
+from base.g import *
+try:
+   import xml.parsers.expat as expat
+except ImportError,e:
+   log.info("\n")
+   log.error("Failed to import xml.parsers.expat(%s).\nThis may be due to the incompatible version of python-xml package.\n"%(e))
+   if "undefined symbol" in str(e):
+       log.info(log.blue("Please re-install compatible version (other than 2.7.2-7.14.1) due to bug reported at 'https://bugzilla.novell.com/show_bug.cgi?id=766778'."))
+       log.info(log.blue("\n        Run the following commands in root mode to change the python-xml package.(i.e Installing 2.7.2-7.1.2)"))
+       log.info(log.blue("\n        Using zypper:\n        'zypper remove python-xml'\n        'zypper install python-xml-2.7.2-7.1.2'"))
+       log.info(log.blue("\n        Using apt-get:\n        'apt-get remove python-xml'\n        'apt-get install python-xml-2.7.2-7.1.2'"))
+       log.info(log.blue("\n        Using yum:\n        'yum remove python-xml'\n        'yum install python-xml-2.7.2-7.1.2'"))
+
+   sys.exit(1)
+
 import re
 import urllib
 try:
@@ -1490,7 +1504,7 @@ def clean(data):
     if data[0] is not '<':
         size = -1
         temp = ""
-        while size: 
+        while size:
             index = data.find('\r\n')
             size = int(data[0:index+1], 16)
             temp = temp + data[index+2:index+2+size]
@@ -1498,19 +1512,21 @@ def clean(data):
         data = temp
     return data
 
-def StatusType10FetchUrl(dev, url, footer=""):
+def StatusType10FetchUrl(func, url, footer=""):
     data_fp = cStringIO.StringIO()
     if footer:
-        data = dev.getEWSUrl_LEDM(url, data_fp, footer)
+        #data = dev.getEWSUrl_LEDM(url, data_fp, footer)
+        data = func(url, data_fp, footer)
     else:
-        data = dev.getEWSUrl_LEDM(url, data_fp)
+        #data = dev.getEWSUrl_LEDM(url, data_fp)
+        data = func(url, data_fp)
         if data:
             data = data.split('\r\n\r\n', 1)[1]
             if data:
                 data = clean(data)
     return data
 
-def StatusType10(dev): # Low End Data Model
+def StatusType10(func): # Low End Data Model
     status_block = { 'revision' :    STATUS_REV_UNKNOWN,
                      'agents' :      [],
                      'top-door' :    TOP_DOOR_NOT_PRESENT,
@@ -1528,7 +1544,7 @@ def StatusType10(dev): # Low End Data Model
         return status_block
 
     # Get the dynamic consumables configuration
-    data = StatusType10FetchUrl(dev, "/DevMgmt/ConsumableConfigDyn.xml")
+    data = StatusType10FetchUrl(func, "/DevMgmt/ConsumableConfigDyn.xml")
     if not data:
         return status_block
     data = data.replace("ccdyn:", "").replace("dd:", "")
@@ -1554,20 +1570,32 @@ def StatusType10(dev): # Low End Data Model
                     if state != "missing":
                         try:
                            ink_level = int(e.find("ConsumablePercentageLevelRemaining").text)
+                           if ink_level == 0:
+                               state = "empty"
+                           elif ink_level <=10:
+                               state = "low"
                         except:
                            ink_level = 0
+                elif type == "printhead":
+                     continue; #No need of adding this agent.
                 else:
                     ink_type = ''
                     if state == "ok":
                         ink_level = 100
+                
+                try:
+                    agent_sku = e.find("ConsumableSelectibilityNumber").text
+                except:
+                    agent_sku = ''
+
+                log.debug("type '%s' state '%s' ink_type '%s' ink_level %d agent_sku = %s" % (type, state, ink_type, ink_level,agent_sku))
 
-                log.debug("type '%s' state '%s' ink_type '%s' ink_level %d" % (type, state, ink_type, ink_level))
-    
                 entry = { 'kind' : element_type10_xlate.get(type, AGENT_KIND_NONE),
                           'type' : pen_type10_xlate.get(ink_type, AGENT_TYPE_NONE),
                           'health' : pen_health10_xlate.get(state, AGENT_HEALTH_OK),
                           'level' : int(ink_level),
-                          'level-trigger' : pen_level10_xlate.get(state, AGENT_LEVEL_TRIGGER_SUFFICIENT_0)
+                          'level-trigger' : pen_level10_xlate.get(state, AGENT_LEVEL_TRIGGER_SUFFICIENT_0),
+                          'agent-sku' : agent_sku
                         }
 
                 log.debug("%s" % entry)
@@ -1579,7 +1607,7 @@ def StatusType10(dev): # Low End Data Model
     status_block['agents'] = agents
 
     # Get the media handling configuration
-    data = StatusType10FetchUrl(dev, "/DevMgmt/MediaHandlingDyn.xml")
+    data = StatusType10FetchUrl(func, "/DevMgmt/MediaHandlingDyn.xml")
     if not data:
         return status_block
     data = data.replace("mhdyn:", "").replace("dd:", "")
@@ -1613,7 +1641,7 @@ def StatusType10(dev): # Low End Data Model
             status_block['duplexer'] = DUPLEXER_DOOR_CLOSED
 
     # Get the product status
-    data = StatusType10FetchUrl(dev, "/DevMgmt/ProductStatusDyn.xml")
+    data = StatusType10FetchUrl(func, "/DevMgmt/ProductStatusDyn.xml")
     if not data:
         return status_block
     data = data.replace("psdyn:", "").replace("locid:", "")
@@ -1690,4 +1718,48 @@ def StatusType10(dev): # Low End Data Model
         elif e.text == "missingPrintHead":
             status_block['status-code'] = STATUS_PRINTER_PRINTHEAD_MISSING
 
+
+		#Alert messages for Pentane products RQ 8888
+        elif e.text == "scannerADFMispick":
+            status_block['status-code'] = STATUS_SCANNER_ADF_MISPICK
+			
+        elif e.text == "mediaTooShortToAutoDuplex":
+            status_block['status-code'] = STATUS_PRINTER_PAPER_TOO_SHORT_TO_AUTODUPLEX
+			
+        elif e.text == "insertOrCloseTray": 
+            status_block['status-code'] = STATUS_PRINTER_TRAY_2_3_DOOR_OPEN
+
+        elif e.text == "inkTooLowToPrime":
+            status_block['status-code'] = STATUS_PRINTER_INK_TOO_LOW_TO_PRIME
+			
+        elif e.text == "cartridgeVeryLow":
+            status_block['status-code'] = STATUS_PRINTER_VERY_LOW_ON_INK
+			
+        elif e.text == "wasteMarkerCollectorAlmostFull":
+            status_block['status-code'] = STATUS_PRINTER_SERVICE_INK_CONTAINER_ALMOST_FULL
+			
+        elif e.text == "wasteMarkerCollectorFull":
+            status_block['status-code'] = STATUS_PRINTER_SERVICE_INK_CONTAINER_FULL			
+			
+        elif e.text == "wasteMarkerCollectorFullPrompt":
+            status_block['status-code'] = STATUS_PRINTER_SERVICE_INK_CONTAINER_FULL_PROMPT
+			
+        elif e.text == "missingDuplexer":
+            status_block['status-code'] = STATUS_PRINTER_DUPLEX_MODULE_MISSING
+
+        elif e.text == "printBarStall":
+            status_block['status-code'] = STATUS_PRINTER_PRINTHEAD_JAM
+
+        elif e.text == "outputBinClosed":
+            status_block['status-code'] = STATUS_PRINTER_CLEAR_OUTPUT_AREA
+
+        elif e.text == "outputBinOpened":
+            status_block['status-code'] = STATUS_PRINTER_CLEAR_OUTPUT_AREA
+
+        elif e.text == "reseatDuplexer":
+            status_block['status-code'] = STATUS_PRINTER_RESEAT_DUPLEXER
+
+        elif e.text == "unexpectedTypeInTray":
+            status_block['status-code'] = STATUS_PRINTER_MEDIA_TYPE_MISMATCH
+
     return status_block
