@@ -16,7 +16,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# Author: Don Welch
+# Author: Don Welch, Naga Samrat Chowdary Narla, Yashwant Kumar Sahu
 #
 
 # string_table := { 'string_id' : 'short', 'long' ), ... }
@@ -96,6 +96,28 @@ class StringTable:
             '1043' : (self.__tr('Custom media mismatch'), '%try_again%'),
             '1044' : (self.__tr('Pen cleaning in progress'), ''),
             '1045' : (self.__tr('Pen checking in progress'), ''),
+            '1046' : (self.__tr('In power save mode'), ''),
+            '1047' : (self.__tr('Incorrect cartridge'), ''),
+            '1048' : (self.__tr('Missing cartridge'), ''),
+            '1049' : (self.__tr('Missing Printhead(s)'), ''),   
+
+			#Alert messages for Pentane products RQ 8888
+            '1050' : (self.__tr('ADF can not load original, please try reloading'), ''),
+            '1051' : (self.__tr('Paper too short to auto duplex'), ''),
+            '1052' : (self.__tr('Tray 2/3 missing or door open'), ''),
+            '1053' : (self.__tr('Ink too low to complete startup process'), ''),
+            '1054' : (self.__tr('Very low on ink'), ''),
+            '1055' : (self.__tr('Service ink container in duplex module in filled almost to its capacity'), self.__tr('Consider replacing duplex module from HP')),
+            '1056' : (self.__tr('Service ink container in duplex module in filled very close to its capacity'), self.__tr('Consider replacing duplex module from HP')),
+            '1057' : (self.__tr('Service ink container in duplex module is full'), self.__tr('Consider replacing duplex module from HP')),
+            '1058' : (self.__tr('Duplex module missing'), ''),
+            '1059' : (self.__tr('Printhead jam'), ''),
+            '1060' : (self.__tr('Remove paper and other material from output flap area'), ''),
+            '1061' : (self.__tr('To Maintain print quality, remove and re-install duplex module'), ''),
+            '1062' : (self.__tr('Media type mismatch'), '%try_again%'),
+            '1063' : (self.__tr('Printhead(s) Failed'), ''),   
+            '1064' : (self.__tr('Incompatible Printhead(s)'), ''),   
+         
             '1501' : (self.__tr('Black cartridge is low on ink'), ''),
             '1502' : (self.__tr('Tri-color cartridge is low on ink'), ''),
             '1503' : (self.__tr('Photo cartridge is low on ink'), ''),
@@ -124,12 +146,28 @@ class StringTable:
             '2001' : (self.__tr('Scan job completed'), ''),
             '2002' : (self.__tr('Scan job failed'), '%try_again%'),
             '2003' : (self.__tr("Scan job failed - Required plug-in not found"), '%502%'),
+            '2004' : (self.__tr('Scanner automatic document feeder is loaded'), ''),
+            '2005' : (self.__tr('Scan to a destination is not specified'), ''),
+            '2006' : (self.__tr('Scanner is waiting for PC'), ''),
+            '2007' : (self.__tr('Scanner automatic document feeder jam'), ''),
+            '2008' : (self.__tr('Scanner automatic document feeder door opened'), ''),
+            '2009' : (self.__tr('Scan job cancelled'), ''),
+            '2010' : (self.__tr('Check scan image size requirements'), ''),
+            '2011' : (self.__tr('Scanner ADF empty'), ''),
+            '2012' : (self.__tr('Scanner ADF mispick'), ''),
+            '2013' : (self.__tr('Scanner busy'), ''),
             '3000' : (self.__tr('Fax job started'), ''),
             '3001' : (self.__tr('Fax job complete'), ''),
             '3002' : (self.__tr('Fax job failed'), '%try_again%'),
             '3003' : (self.__tr('Fax job canceled'), ''),
             '3004' : (self.__tr('Fax send job continuing'), ''),
             '3005' : (self.__tr('Fax receive job continuing'), ''),
+            '3006' : (self.__tr('Fax dialing'), ''),
+            '3007' : (self.__tr('Fax connecting'), ''),
+            '3008' : (self.__tr('Fax send error'), ''),
+            '3009' : (self.__tr('Fax error storage full'), ''),
+            '3010' : (self.__tr('Fax receive error'), ''),
+            '3011' : (self.__tr('Fax blocking'), ''),
             '4000' : (self.__tr('Copy job started'), ''),
             '4001' : (self.__tr('Copy job complete'), ''),
             '4002' : (self.__tr('Copy job failed'), '%try_again%'),
@@ -141,6 +179,7 @@ class StringTable:
             '5026' : (self.__tr("Device status not available"), ''),
             '5031' : ('%5021%', ''),
             '5034' : (self.__tr('Device does not support requested operation'), '%try_again%'),
+            '5501' : (self.__tr('Device added'), ''),
             '6000' : (self.__tr('Photocard unload started'), ''),
             '6001' : (self.__tr('Photocard unload ended'), ''),
             '6002' : (self.__tr('Photocard unload failed'), self.__tr('Make sure photocard is inserted properly and try again')),
