@@ -16,7 +16,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Author: Don Welch, Naga Samrat Chowdary Narla
+# Author: Don Welch, Naga Samrat Chowdary Narla, Goutam Kodu, Amarnath Chitumalla
 #
 # Thanks to Henrique M. Holschuh <hmh@debian.org> for various security patches
 #
@@ -26,6 +26,7 @@ from __future__ import generators
 # Std Lib
 import sys
 import os
+from subprocess import Popen, PIPE
 import grp
 import fnmatch
 import tempfile
@@ -37,13 +38,29 @@ import fcntl
 import errno
 import stat
 import string
+import glob
 import commands # TODO: Replace with subprocess (commands is deprecated in Python 3.0)
 import cStringIO
 import re
-import xml.parsers.expat as expat
+from base.g import *
+try:
+   import xml.parsers.expat as expat
+except ImportError,e:
+   log.info("\n")
+   log.error("Failed to import xml.parsers.expat(%s).\nThis may be due to the incompatible version of python-xml package.\n"%(e))
+   if "undefined symbol" in str(e):
+       log.info(log.blue("Please re-install compatible version (other than 2.7.2-7.14.1) due to bug reported at 'https://bugzilla.novell.com/show_bug.cgi?id=766778'."))
+       log.info(log.blue("\n        Run the following commands in root mode to change the python-xml package.(i.e Installing 2.7.2-7.1.2)"))
+       log.info(log.blue("\n        Using zypper:\n        'zypper remove python-xml'\n        'zypper install python-xml-2.7.2-7.1.2'"))
+       log.info(log.blue("\n        Using apt-get:\n        'apt-get remove python-xml'\n        'apt-get install python-xml-2.7.2-7.1.2'"))
+       log.info(log.blue("\n        Using yum:\n        'yum remove python-xml'\n        'yum install python-xml-2.7.2-7.1.2'"))
+
+   sys.exit(1)
+
 import getpass
 import locale
 import htmlentitydefs
+import urllib
 
 try:
     import platform
@@ -51,6 +68,15 @@ try:
 except ImportError:
     platform_avail = False
 
+try:
+    import dbus
+    from dbus import SystemBus, lowlevel
+    dbus_avail=True
+except ImportError:
+    dbus_avail=False
+
+
+
 # Local
 from g import *
 from codes import *
@@ -58,6 +84,7 @@ import pexpect
 
 BIG_ENDIAN = 0
 LITTLE_ENDIAN = 1
+DBUS_SERVICE='com.hplip.StatusService'
 
 def addgroup():
     lis = []
@@ -544,6 +571,11 @@ class UserSettings(object): # Note: Deprecated after 2.8.8 in Qt4 (see ui4/ui_ut
         self.cmd_copy = user_conf.get('commands', 'cpy', self.cmd_copy)
         self.cmd_fax = user_conf.get('commands', 'fax', self.cmd_fax)
         self.cmd_fab = user_conf.get('commands', 'fab', self.cmd_fab)
+
+        self.upgrade_notify= to_bool(user_conf.get('upgrade', 'notify_upgrade', '0'))
+        self.upgrade_last_update_time = int(user_conf.get('upgrade','last_upgraded_time', '0'))
+        self.upgrade_pending_update_time =int(user_conf.get('upgrade', 'pending_upgrade_time', '0'))
+        self.latest_available_version=str(user_conf.get('upgrade', 'latest_available_version',''))
         self.debug()
 
     def debug(self):
@@ -556,6 +588,10 @@ class UserSettings(object): # Note: Deprecated after 2.8.8 in Qt4 (see ui4/ui_ut
         log.debug("Auto refresh: %s" % self.auto_refresh)
         log.debug("Auto refresh rate: %s" % self.auto_refresh_rate)
         log.debug("Auto refresh type: %s" % self.auto_refresh_type)
+        log.debug("Upgrade notification:%d"  %self.upgrade_notify)
+        log.debug("Last Installed time:%d" %self.upgrade_last_update_time)
+        log.debug("Next scheduled installation time:%d" % self.upgrade_pending_update_time)
+
 
     def save(self):
         log.debug("Saving user settings...")
@@ -567,6 +603,11 @@ class UserSettings(object): # Note: Deprecated after 2.8.8 in Qt4 (see ui4/ui_ut
         user_conf.set('refresh', 'enable',self.auto_refresh)
         user_conf.set('refresh', 'rate', self.auto_refresh_rate)
         user_conf.set('refresh', 'type', self.auto_refresh_type)
+        user_conf.set('upgrade', 'notify_upgrade', self.upgrade_notify)
+        user_conf.set('upgrade','last_upgraded_time', self.upgrade_last_update_time)
+        user_conf.set('upgrade', 'pending_upgrade_time', self.upgrade_pending_update_time)
+        user_conf.set('upgrade', 'latest_available_version', self.latest_available_version)
+
         self.debug()
 
 
@@ -839,14 +880,15 @@ def all(S,f=lambda x:x):
         if not f(x): return False
     return True
 
-
-BROWSERS = ['firefox', 'mozilla', 'konqueror', 'galeon', 'skipstone'] # in preferred order
-BROWSER_OPTS = {'firefox': '-new-window', 'mozilla' : '', 'konqueror': '', 'galeon': '-w', 'skipstone': ''}
+BROWSERS = ['firefox', 'mozilla', 'konqueror', 'epiphany', 'skipstone'] # in preferred order
+BROWSER_OPTS = {'firefox': '-new-tab', 'mozilla': '', 'konqueror': '', 'epiphany': '--new-tab', 'skipstone': ''}
 
 
 def find_browser():
     if platform_avail and platform.system() == 'Darwin':
         return "open"
+    elif which("xdg-open"):
+        return "xdg-open"
     else:
         for b in BROWSERS:
             if which(b):
@@ -860,11 +902,14 @@ def openURL(url, use_browser_opts=True):
         cmd = 'open "%s"' % url
         log.debug(cmd)
         os.system(cmd)
+    elif which("xdg-open"):
+        cmd = 'xdg-open "%s"' % url
+        log.debug(cmd)
+        os.system(cmd)
     else:
         for b in BROWSERS:
-            bb = which(b)
+            bb = which(b, return_full_path='True')
             if bb:
-                bb = os.path.join(bb, b)
                 if use_browser_opts:
                     cmd = """%s %s "%s" &""" % (bb, BROWSER_OPTS[b], url)
                 else:
@@ -973,11 +1018,114 @@ class XMLToDictParser:
         parser = expat.ParserCreate()
         parser.StartElementHandler = self.startElement
         parser.EndElementHandler = self.endElement
-        parser.CharacterDataHandler = self.charData
-        parser.Parse(text.encode('utf-8'), True)
+        parser.CharacterDataHandler = self.charData        
+        try:
+            parser.Parse(text.encode('utf-8'), True)
+        except UnicodeDecodeError:
+            log.error("Received Unicode error in xml= %s "%text)
         return self.data
 
 
+
+class Element:
+    def __init__(self,name,attributes):
+        self.name = name
+        self.attributes = attributes
+        self.chardata = ''
+        self.children = []
+        
+    def AddChild(self,element):
+        self.children.append(element)
+        
+    def getAttribute(self,key):
+        return self.attributes.get(key)
+    
+    def getData(self):
+        return self.chardata
+        
+    def getElementsByTagName(self,name='',ElementNode=None):
+        if ElementNode:
+            Children_list = ElementNode.children
+        else:
+            Children_list = self.children
+        if not name:
+            return self.children
+        else:
+            elements = []
+            for element in Children_list:
+                if element.name == name:
+                    elements.append(element)
+                
+                rec_elements = self.getElementsByTagName (name,element)
+                for a in rec_elements:
+                    elements.append(a)
+            return elements
+    
+    def getChildElements(self,name=''):
+        if not name:
+            return self.children
+        else:
+            elements = []
+            for element in self.children:
+                if element.name == name:
+                    elements.append(element)
+            return elements
+
+    def toString(self, level=0):
+        retval = " " * level
+        retval += "<%s" % self.name
+        for attribute in self.attributes:
+            retval += " %s=\"%s\"" % (attribute, self.attributes[attribute])
+        c = ""
+        for child in self.children:
+            c += child.toString(level+1)
+        if c == "":
+            if self.chardata:
+                retval += ">"+self.chardata + ("</%s>" % self.name)                
+            else:
+                retval += "/>"
+        else:
+            retval += ">" + c + ("</%s>" % self.name)
+        return retval
+        
+class  extendedExpat:
+    def __init__(self):
+        self.root = None
+        self.nodeStack = []
+        
+    def StartElement_EE(self,name,attributes):
+        element = Element(name.encode(),attributes)
+        
+        if len(self.nodeStack) > 0:
+            parent = self.nodeStack[-1]
+            parent.AddChild(element)
+        else:
+            self.root = element
+        self.nodeStack.append(element)
+        
+    def EndElement_EE(self,name):
+        self.nodeStack = self.nodeStack[:-1]
+
+    def charData_EE(self,data):        
+        if string.strip(data):
+            data = data.encode()
+            element = self.nodeStack[-1]
+            element.chardata += data
+            return
+
+    def Parse(self,xmlString):
+        Parser = expat.ParserCreate()
+
+        Parser.StartElementHandler = self.StartElement_EE
+        Parser.EndElementHandler = self.EndElement_EE
+        Parser.CharacterDataHandler = self.charData_EE
+
+        Parser.Parse(xmlString.encode('utf-8'), True)
+        
+        return self.root
+
+
+
 def dquote(s):
     return ''.join(['"', s, '"'])
 
@@ -1033,7 +1181,7 @@ def getEndian():
 def get_password():
     return getpass.getpass("Enter password: ")
 
-def get_password_ui():
+def get_password_ui(pswd_msg=''):
     fp = open("/etc/hp/hplip.conf", "r")
     qt = "qt3"
     for line in fp:
@@ -1042,13 +1190,19 @@ def get_password_ui():
     fp.close()
     if qt is "qt4":
         from ui4.setupdialog import showPasswordUI
-        username, password = showPasswordUI("Your printer requires to install HP proprietary plugin\nPlease enter root/superuser password to continue")
+        if pswd_msg == '':
+            username, password = showPasswordUI("Your HP Device requires to install HP proprietary plugin\nPlease enter root/superuser password to continue", "root", False)
+        else:
+            username, password = showPasswordUI(pswd_msg, "root", False)
     if qt is "qt3":
         from ui.setupform import showPasswordUI
-        username, password = showPasswordUI("Your priter requires to install HP proprietary plugin\nPlease enter root/superuser password to continue")
-    return password
+        if pswd_msg == '':
+            username, password = showPasswordUI("Your HP Device requires to install HP proprietary plugin\nPlease enter root/superuser password to continue", "root", False)
+        else:
+            username, password = showPasswordUI(pswd_msg, "root", False)
+    return  password
 
-def run(cmd, log_output=True, password_func=get_password, timeout=1):
+def run(cmd, log_output=True, password_func=get_password, timeout=1, spinner=True, pswd_msg=''):
     output = cStringIO.StringIO()
 
     try:
@@ -1058,7 +1212,9 @@ def run(cmd, log_output=True, password_func=get_password, timeout=1):
 
     try:
         while True:
-            update_spinner()
+            if spinner:
+                update_spinner()
+
             i = child.expect(["[pP]assword:", pexpect.EOF, pexpect.TIMEOUT])
 
             if child.before:
@@ -1069,12 +1225,13 @@ def run(cmd, log_output=True, password_func=get_password, timeout=1):
             if i == 0: # Password:
                 if password_func is not None:
                     if password_func == "get_password_ui":
-                        child.sendline(get_password_ui())
+                        child.sendline(get_password_ui(pswd_msg))
                     else:
                         child.sendline(password_func())
                 else:
                     child.sendline(get_password())
-
+	        
+ 
             elif i == 1: # EOF
                 break
 
@@ -1084,8 +1241,8 @@ def run(cmd, log_output=True, password_func=get_password, timeout=1):
 
     except Exception, e:
         log.error("Exception: %s" % e)
-
-    cleanup_spinner()
+    if spinner:
+        cleanup_spinner()
     child.close()
 
     return child.exitstatus, output.getvalue()
@@ -1581,7 +1738,7 @@ encoding: utf8
         log.info("contact the HPLIP Team.")
 
         log.info(".SH COPYRIGHT")
-        log.info("Copyright (c) 2001-9 Hewlett-Packard Development Company, L.P.")
+        log.info("Copyright (c) 2001-14 Hewlett-Packard Development Company, L.P.")
         log.info(".LP")
         log.info("This software comes with ABSOLUTELY NO WARRANTY.")
         log.info("This is free software, and you are welcome to distribute it")
@@ -1600,7 +1757,7 @@ def log_title(program_name, version, show_ver=True): # TODO: Move to base/module
 
     log.info(log.bold("%s ver. %s" % (program_name, version)))
     log.info("")
-    log.info("Copyright (c) 2001-9 Hewlett-Packard Development Company, LP")
+    log.info("Copyright (c) 2001-14 Hewlett-Packard Development Company, LP")
     log.info("This software comes with ABSOLUTELY NO WARRANTY.")
     log.info("This is free software, and you are welcome to distribute it")
     log.info("under certain conditions. See COPYING file for more details.")
@@ -1629,6 +1786,39 @@ def su_sudo():
 
     return su_sudo_str
 
+# This function returns the distro name and distro version. 
+#This is provided to check on Fedora 14 in pkit.py file for Plugin-installation. 
+#is_su variable is used to provide a check on Fedora 8 
+def os_release():
+    os_name = None;
+    os_version = None;
+    is_su = None;
+    if which('lsb_release'):
+       name = os.popen('lsb_release -i | cut -f 2')
+       os_name = name.read().strip()
+       name.close()
+       version = os.popen('lsb_release -r | cut -f 2')
+       os_version=version.read().strip()
+       version.close()
+       is_su = True
+    else:
+       name = os.popen('cat /etc/issue | cut -c 1-6 | head -n 1')
+       os_name = name.read().strip()
+       name.close()
+       version1=os.popen('cat /etc/issue | cut -c 16 | head -n 1')
+       version2=version1.read().strip()
+       version1.close()
+       if (version2 == '1'):
+           version=os.popen('cat /etc/issue | cut -c 16-17 | head -n 1')
+           is_su = True
+       else: 
+           version=os.popen('cat /etc/issue | cut -c 16 | head -n 1')
+           is_su = False
+       os_version=version.read().strip()
+       version.close()
+
+    return os_name,os_version,is_su 
+    
 
 #
 # Removes HTML or XML character references and entities from a text string.
@@ -1675,3 +1865,198 @@ def escape(s):
         s = s.replace(unichr(c), u"&#%d;" % c)
 
     return s
+
+# checks if given process is running.
+#return value:
+#    True or False
+#    None - if process is not running
+#    grep output - if process is running
+
+def Is_Process_Running(process_name):
+    try:
+        p1 = Popen(["ps", "aux"], stdout=PIPE)
+        p2 = Popen(["grep", process_name], stdin=p1.stdout, stdout=PIPE)
+        p3 = Popen(["grep", "-v", "grep"], stdin=p2.stdout, stdout=PIPE)
+        output = p3.communicate()[0]
+        log.debug("Is_Process_Running outpu = %s " %output)
+
+        if process_name in output:
+            return True, output
+        else:
+            return False, None
+
+    except Exception, e:
+        log.error("Execution failed: process Name[%s]" %process_name)
+        print >>sys.stderr, "Execution failed:", e
+        return False, None
+
+#return tye: strings
+#Return values.
+#   None --> on error.
+#  "terminal name"-->success
+def get_terminal():
+    terminal_list=['gnome-terminal', 'konsole','x-terminal-emulator', 'xterm', 'gtkterm']
+    cnt = 0
+    terminal_cmd = None
+    while cnt < len(terminal_list):
+        if which(terminal_list[cnt]):
+            terminal_cmd = terminal_list[cnt]+" -e "
+            log.debug("Available Terminal = %s " %terminal_cmd)
+            break
+            
+    return terminal_cmd
+
+#Return Type: bool
+# Return values:
+#      True --> if it is older version
+#      False  --> if it is same or later version.
+
+def Is_HPLIP_older_version(installed_version, available_version):
+    
+    if available_version == "" or available_version == None or installed_version == "" or installed_version == None:
+        log.debug("available_version is ''")
+        return False
+
+    installed_array=installed_version.split('.')
+    available_array=available_version.split('.')
+
+    log.debug("HPLIP Installed_version=%s  Available_version=%s"%(installed_version,available_version))
+    cnt = 0
+    Is_older = False
+    while cnt <len(installed_array) and cnt <len(available_array):
+        if(int(installed_array[cnt]) < int(available_array[cnt])):
+            Is_older = True
+            break
+        elif(int(installed_array[cnt]) > int(available_array[cnt])):
+            log.debug("Already new verison is installed")
+            return False
+        cnt += 1
+
+    # To check internal version is installed.
+    if Is_older is False and len(installed_array) >len(available_array):
+        Is_older = True
+
+    return Is_older
+
+
+def downLoad_status(count, blockSize, totalSize):
+    percent = int(count*blockSize*100/totalSize)
+    if count != 0:
+        sys.stdout.write("\b\b\b")
+    sys.stdout.write("%s" %(log.color("%2d%%"%percent, 'bold')))
+    sys.stdout.flush()
+
+
+def download_from_network(weburl, outputFile = None, useURLLIB=False):
+    result =False
+
+    if weburl is "" or weburl is None:
+        log.error("URL is empty")
+        return result, ""
+
+    if outputFile is None:
+        fp, outputFile = make_temp_file()
+
+    try:
+        if useURLLIB is False:
+            wget = which("wget")
+            if wget:
+                wget = os.path.join(wget, "wget")
+                status, output = run("%s --cache=off --timeout=60 --output-document=%s %s" %(wget, outputFile, weburl))
+                if status:
+                    log.error("Failed to connect to HPLIP site. Error code = %d" %status)
+                    return False, ""
+            else:
+                useURLLIB = True
+
+        if useURLLIB:
+            sys.stdout.write("Download in progress...")
+            urllib.urlretrieve(weburl, outputFile, downLoad_status)
+
+    except IOError, e:
+        log.error("I/O Error: %s" % e.strerror)
+        return False, ""
+
+    if not os.path.exists(outputFile):
+        log.error("Failed to get hplip version/ %s file not found."%hplip_version_file)
+        return False, ""
+
+    return True, outputFile
+
+
+
+
+    
+class Sync_Lock:
+    def __init__(self, filename):
+        self.Lock_filename = filename
+        self.handler = open(self.Lock_filename, 'w')
+
+# Wait for another process to release resource and acquires the resource.
+    def acquire(self):
+        fcntl.flock(self.handler, fcntl.LOCK_EX)
+
+    def release(self):
+        fcntl.flock(self.handler, fcntl.LOCK_UN)
+
+    def __del__(self):
+        self.handler.close()
+
+def sendEvent(event_code,device_uri, printer_name, username="", job_id=0, title="", pipe_name=''):
+    
+    if not dbus_avail:
+        log.debug("Failed to import dbus, lowlevel")
+        return
+        
+    log.debug("send_message() entered")
+    args = [device_uri, printer_name, event_code, username, job_id, title, pipe_name]
+    msg = lowlevel.SignalMessage('/', DBUS_SERVICE, 'Event')
+    msg.append(signature='ssisiss', *args)
+    SystemBus().send_message(msg)
+    log.debug("send_message() returning")
+
+def expand_list(File_exp):
+   File_list = glob.glob(File_exp)
+   if File_list:
+      File_list_str = ' '.join(File_list)
+      return File_list, File_list_str
+   else:
+      return [],""
+
+
+
+def unchunck_xml_data(src_data):
+    index = 0
+    dst_data=""
+    # src_data contains HTTP data + xmlpayload. delimter is '\r\n\r\n'.
+    if src_data.find('\r\n\r\n') != -1:
+        src_data = src_data.split('\r\n\r\n', 1)[1]
+    else:
+        return dst_data
+    
+    if len(src_data) <= 0:
+        return dst_data
+
+    #If xmlpayload doesn't have chuncksize embedded, returning same xml.
+    if src_data[index] == '<':
+        dst_data = src_data
+    else:  # Removing chunck size from xmlpayload
+        try:
+           while index < len(src_data):
+             buf_len = 0
+             while src_data[index] == ' ' or src_data[index] == '\r' or src_data[index] == '\n':
+               index = index +1
+             while src_data[index] != '\n' and src_data[index] != '\r':
+               buf_len = buf_len *16 + int(src_data[index], 16)
+               index = index +1
+             
+             if buf_len == 0:
+                 break;
+
+             dst_data = dst_data+ src_data[index:buf_len+index+2]
+
+             index = buf_len + index + 2  # 2 for after size '\r\n' chars.
+        except IndexError:
+            pass
+    return dst_data
+
