@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# (c) Copyright 2003-2009 Hewlett-Packard Development Company, L.P.
+# (c) Copyright 2003-2014 Hewlett-Packard Development Company, L.P.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Author: Don Welch
+# Author: Don Welch, Amarnath Chitumalla
 #
 
 # Std Lib
@@ -30,6 +30,22 @@ import signal
 from base.g import *
 from base import utils, tui
 from core_install import *
+from prnt import cups
+
+def start_systray():
+    tui.title("RE-STARTING HP_SYSTRAY")
+    path = utils.which('hp-systray')
+    if path:
+        path = os.path.join(path, 'hp-systray')
+    else:
+        path = os.path.join(prop.home_dir, 'systray.py')
+    if not os.path.exists(path):
+        log.warn("Unable to start hp-systray")
+
+    log.debug("Running hp-systray: %s --force-startup" % path)
+    os.spawnlp(os.P_NOWAIT, path, 'hp-systray', '--force-startup')
+    log.debug("Waiting for hp-systray to start...")
+    time.sleep(1)
 
 
 def progress_callback(cmd="", desc="Working..."):
@@ -42,6 +58,9 @@ def progress_callback(cmd="", desc="Working..."):
 def password_entry():
     return getpass.getpass(log.bold("Please enter the root/superuser password: "))
 
+def password_user_entry():
+    return getpass.getpass(log.bold("Please enter the user (%s)'s password: " % os.getenv('USER')))
+
 
 def option_question_callback(opt, desc, default='y'):
     ok, ans = tui.enter_yes_no("Do you wish to enable '%s'" % desc, default)
@@ -73,6 +92,33 @@ def start(language, auto=True, test_depends=False,
 
         log.info("")
         log.note("Defaults for each question are maked with a '*'. Press <enter> to accept the default.")
+        core.init()
+        vrs =core.get_distro_data('versions_list')
+        Is_Manual_Distro = False
+        distro_alternate_version=None
+        if core.distro_version not in vrs and len(vrs):
+            distro_alternate_version= vrs[len(vrs)-1]
+            if core.is_auto_installer_support(distro_alternate_version):
+                log.error("%s-%s version is not supported, so all dependencies may not be installed. However trying to install using %s-%s version packages." \
+                                                   %(core.distro_name, core.distro_version, core.distro_name, distro_alternate_version))
+                ok, choice = tui.enter_choice("\nPress 'y' to continue auto installation. Press 'n' to quit auto instalation(y=yes, n=no*): ",['y','n'],'n')
+                if not ok or choice =='n':
+                    log.info("Installation exit")
+                    sys.exit()
+            else:
+                # Even previous distro is not supported
+                Is_Manual_Distro = True
+        elif not core.is_auto_installer_support():
+              # This distro is not supported
+            Is_Manual_Distro = True
+
+
+        if Is_Manual_Distro:
+            log.error("Auto installation is not supported for '%s' distro so all dependencies may not be installed. \nPlease install manually as mentioned in 'http://hplipopensource.com/hplip-web/install/manual/index.html' web-site"% core.distro_name)
+            ok, choice = tui.enter_choice("\nPress 'y' to continue auto installation. Press 'n' to quit auto instalation(y=yes, n=no*): ",['y','n'],'n')
+            if not ok or choice =='n':
+                log.info("Installation exit")
+                sys.exit()
 
         if not auto:
             tui.title("INSTALLATION MODE")
@@ -99,7 +145,8 @@ def start(language, auto=True, test_depends=False,
                 return
 
         log.info("\nInitializing. Please wait...")
-        core.init()
+        prev_hplip_version= sys_conf.get("hplip","version","0.0.0")
+
 
         if test_unknown:
             core.distro_name = 'unknown'
@@ -108,21 +155,8 @@ def start(language, auto=True, test_depends=False,
 
 
         #
-        # HPLIP vs. HPIJS INSTALLATION
+        # HPLIP INSTALLATION
         #
-
-        #if not auto:
-            #tui.title("INSTALL TYPE")
-            #log.info("For most users, it is recommended to install HPLIP with full support (scanning, faxing, toolbox, etc).")
-            #log.info("For servers or minimal installations, you can also install print support only (HPIJS only).")
-
-            #ok, choice = tui.enter_choice("\nInstall full hplip support (recommended) or print-only support (f=full hplip support*, p=printing only support, q=quit) ?",
-                #['f', 'p'], 'f')
-            #if not ok: sys.exit(0)
-            #if choice  == 'p':
-                #core.selected_component = 'hpijs'
-
-        #log.debug(core.selected_component)
         core.selected_component = 'hplip'
 
         #
@@ -133,31 +167,9 @@ def start(language, auto=True, test_depends=False,
 
         if core.selected_component == 'hplip':
             log.info("This installer will install HPLIP version %s on your computer." % core.version_public)
-            #core.hpijs_build = False
-        #else:
-            #log.info("This installer will install HPIJS version %s on your computer." % core.version_public)
-            #core.hpijs_build = True
 
         log.info("Please close any running package management systems now (YaST, Adept, Synaptic, Up2date, etc).")
 
-##        #
-##        # RELEASE NOTES
-##        #
-##
-##        if not auto:
-##            if os.getenv('DISPLAY'):
-##                tui.title("VIEW RELEASE NOTES")
-##                log.info("Release notes from this version are available as a web (HTML) document.")
-##                log.info("The release notes file will be shown in a separate web browser window.")
-##
-##                ok, ans = tui.enter_yes_no("\nWould you like to view the release notes for this version of HPLIP", 'n')
-##
-##                if ok and ans:
-##                    log.info("Displaying release notes in a browser window...")
-##                    core.show_release_notes_in_browser()
-##
-##                if not ok:
-##                    sys.exit(0)
 
         # For testing, mark all dependencies missing
         if test_depends:
@@ -294,7 +306,8 @@ def start(language, auto=True, test_depends=False,
                     (core.get_distro_data('display_name', '(unknown)'), core.distro_version))
 
 
-            if core.distro == DISTRO_UNKNOWN or not core.distro_version_supported:
+#            if core.distro == DISTRO_UNKNOWN or not core.distro_version_supported:
+            if core.distro == DISTRO_UNKNOWN:
                 log.error("The distribution/OS that you are running is not supported. This installer\ncannot install an unsupported distribution. Please check your distribution/OS\nand re-run this installer or perform a manual installation.")
                 if num_req_missing:
                     log.error("The following REQUIRED dependencies are missing and need to be installed:")
@@ -310,53 +323,46 @@ def start(language, auto=True, test_depends=False,
 
                 sys.exit(1)
 
+
         #
         # SELECT OPTIONS TO INSTALL
         #
 
-
-
         if not auto:
             tui.title("SELECT HPLIP OPTIONS")
             log.info("You can select which HPLIP options to enable. Some options require extra dependencies.")
             log.info("")
             num_opt_missing = core.select_options(option_question_callback)
 
-        #else: # auto
-            #ok, enable_par = tui.enter_yes_no("Would you like to enable support for parallel (LPT:) connected printers?", 'n')
-            #if not ok: sys.exit(0)
-
-            #core.selected_options['parallel'] = enable_par
-
-            #if enable_par:
-                #log.info("Parallel support enabled.")
         else:
             enable_par = False
             core.selected_options['parallel'] = False
 
-        log.debug("Req missing=%d Opt missing=%d HPLIP=%s Component=%s" % \
-            (num_req_missing, num_opt_missing, core.hplip_present, core.selected_component))
 
 
         #
         # COLLECT SUPERUSER PASSWORD
         #
         if not core.running_as_root():
-            tui.title("ENTER ROOT/SUPERUSER PASSWORD")
-
-            ok = core.check_password(password_entry, progress_callback)
+            su_sudo = core.get_distro_data('su_sudo')
+            if su_sudo == "sudo":
+                tui.title("ENTER USER PASSWORD")
+                ok = core.check_password(password_user_entry, progress_callback)
+            else:
+                tui.title("ENTER ROOT/SUPERUSER PASSWORD")
+                ok = core.check_password(password_entry, progress_callback)
 
             if not ok:
-                log.error("3 incorrect attempts. Exiting.")
+                log.error("3 incorrect attempts. (or) Insufficient permissions(i.e. try with sudo user).\nExiting.")
                 sys.exit(1)
 
-        #
+
         # INSTALLATION NOTES
         #
 
-        if core.distro_supported():
+        if core.is_auto_installer_support(distro_alternate_version):
             distro_notes = core.get_distro_data('notes', '').strip()
-            ver_notes = core.get_ver_data('notes', '').strip()
+            ver_notes = core.get_ver_data('notes', '',distro_alternate_version).strip()
 
             if distro_notes or ver_notes:
                 tui.title("INSTALLATION NOTES")
@@ -376,7 +382,7 @@ def start(language, auto=True, test_depends=False,
         # PRE-INSTALL COMMANDS
         #
         tui.title("RUNNING PRE-INSTALL COMMANDS")
-        if core.run_pre_install(progress_callback): # some cmds were run...
+        if core.run_pre_install(progress_callback, distro_alternate_version): # some cmds were run...
             num_req_missing = core.count_num_required_missing_dependencies()
             num_opt_missing = core.count_num_optional_missing_dependencies()
         log.info("OK")
@@ -384,7 +390,7 @@ def start(language, auto=True, test_depends=False,
         #
         # REQUIRED DEPENDENCIES INSTALL
         #
-
+        package_mgr_cmd = core.get_distro_data('package_mgr_cmd')
         depends_to_install = []
         if num_req_missing:
             tui.title("INSTALL MISSING REQUIRED DEPENDENCIES")
@@ -396,11 +402,12 @@ def start(language, auto=True, test_depends=False,
                 log.warning("Missing REQUIRED dependency: %s (%s)" % (depend, desc))
 
                 ok = False
-                packages, commands = core.get_dependency_data(depend)
+                packages, commands = core.get_dependency_data(depend,distro_alternate_version)
                 log.debug("Packages: %s" % ','.join(packages))
                 log.debug("Commands: %s" % ','.join(commands))
 
-                if core.distro_version_supported and (packages or commands):
+#                if core.distro_version_supported and (packages or commands):
+                if package_mgr_cmd and (packages or commands):
                     if auto:
                         answer = True
                     else:
@@ -419,9 +426,6 @@ def start(language, auto=True, test_depends=False,
                     log.error("Installation cannot continue without this dependency. Please manually install this dependency and re-run this installer.")
                     sys.exit(0)
 
-                #log.info("-"*10)
-                #log.info("")
-
         #
         # OPTIONAL dependencies
         #
@@ -441,12 +445,13 @@ def start(language, auto=True, test_depends=False,
                     log.warning("Missing OPTIONAL dependency for option '%s': %s (%s)" % (opt, depend, desc))
 
                 installed = False
-                packages, commands = core.get_dependency_data(depend)
+                packages, commands = core.get_dependency_data(depend,distro_alternate_version)
                 log.debug("Packages: %s" % ','.join(packages))
                 log.debug("Commands: %s" % ','.join(commands))
 
 
-                if core.distro_version_supported and (packages or commands):
+#                if core.distro_version_supported and (packages or commands):
+                if package_mgr_cmd and (packages or commands):
                     if auto:
                         answer = True
                     else:
@@ -472,16 +477,16 @@ def start(language, auto=True, test_depends=False,
                         core.selected_options[opt] = False
 
 
-                #log.info("-"*10)
-                #log.info("")
 
+        log.debug("Dependencies to install: %s  hplip_present:%s" % (depends_to_install, core.hplip_present))
 
-        log.debug("Dependencies to install: %s" % depends_to_install)
+#        if core.distro_version_supported and \
+#            (depends_to_install or core.hplip_present) and \
+#            core.selected_component == 'hplip':
 
-        if core.distro_version_supported and \
+        if package_mgr_cmd and \
             (depends_to_install or core.hplip_present) and \
             core.selected_component == 'hplip':
-
             #
             # CHECK FOR RUNNING PACKAGE MANAGER
             #
@@ -515,14 +520,17 @@ def start(language, auto=True, test_depends=False,
             #
             # CHECK FOR ACTIVE NETWORK CONNECTION
             #
-
             if not assume_network:
                 tui.title("CHECKING FOR NETWORK CONNECTION")
 
                 if not core.check_network_connection():
-                    log.error("\nThe network appears to be unreachable. Installation cannot complete without access to")
-                    log.error("distribution repositories. Please check the network and try again.")
-                    sys.exit(1)
+                    log.error("The network appears to be unreachable. Installation may not resolve all dependencies without access to distribution repositories.")
+                    ok, choice = tui.enter_choice("Do you want to continue installation without network?. Press 'y' for YES. Press 'n' for NO (y=yes*, n=no) : ",['y', 'n'], 'y')
+                    if not ok or choice == 'n':
+                        log.info("Please connect network and try again")
+                        sys.exit(1)
+                    else:
+                        log.debug("Continuing installation without network")
                 else:
                     log.info("Network connection present.")
 
@@ -531,7 +539,7 @@ def start(language, auto=True, test_depends=False,
             #
 
             tui.title("RUNNING PRE-PACKAGE COMMANDS")
-            core.run_pre_depend(progress_callback)
+            core.run_pre_depend(progress_callback,distro_alternate_version)
             log.info("OK")
 
             #
@@ -554,7 +562,7 @@ def start(language, auto=True, test_depends=False,
 
                 for d in depends_to_install:
                     log.debug("*** Processing dependency: %s" % d)
-                    pkgs, commands = core.get_dependency_data(d)
+                    pkgs, commands = core.get_dependency_data(d,distro_alternate_version)
 
                     if pkgs:
                         log.debug("Package(s) '%s' will be installed to satisfy dependency '%s'." %
@@ -641,71 +649,35 @@ def start(language, auto=True, test_depends=False,
                         sys.exit(1)
 
 
-            ##
-            ## HPOJ REMOVAL
-            ##
-
-            #if core.hpoj_present and core.selected_component == 'hplip' and core.distro_version_supported:
-                #log.error("HPOJ is installed and/or running. HPLIP is not compatible with HPOJ.")
-                #failed = True
-                #hpoj_remove_cmd = core.get_distro_data('hpoj_remove_cmd')
-
-                #if hpoj_remove_cmd:
-                    #if auto:
-                        #answer = True
-                    #else:
-                        #ok, answer = tui.enter_yes_no("\nWould you like to have this installer attempt to uninstall HPOJ")
-
-                    #if not ok: sys.exit(0)
-
-                    #if answer:
-                        #failed = core.remove_hpoj(progress_callback)
-
-                        #if failed:
-                            #log.error("HPOJ removal failed. Please manually stop/remove/uninstall HPOJ and then re-run this installer.")
-                            #sys.exit(1)
-                    #else:
-                        #log.error("Please stop/remove/uninstall HPOJ and then re-run this installer.")
-                        #sys.exit(1)
-
-                #else:
-                    #log.error("Please stop/remove/uninstall HPOJ and then re-run this installer.")
-                    #sys.exit(1)
 
 
             #
             # HPLIP REMOVE
             #
+            num_req_missing = 0
+            core.check_dependencies()
+            for depend, desc, opt in core.missing_required_dependencies():
+                log.error("A required dependency '%s (%s)' is still missing." % (depend, desc))
+                num_req_missing += 1
 
-            if core.hplip_present and core.selected_component == 'hplip' and core.distro_version_supported:
-                failed = True
-                log.warn("A previous install of HPLIP is installed and/or running.")
-
-                hplip_remove_cmd = core.get_distro_data('hplip_remove_cmd')
-                if hplip_remove_cmd:
-                    if auto:
-                        answer = True
-                    else:
-                        ok, answer = tui.enter_yes_no("\nWould you like to have this installer attempt to uninstall the previously installed HPLIP")
-                    if not ok: sys.exit(0)
-
-                    if answer:
-                        failed = core.remove_hplip(progress_callback)
-
-                else:
-                    log.error("The previously installed version of HPLIP may conflict with the new one being installed.")
-                    log.error("It is recommended that you quit this installer, and manually remove HPLIP before continuing.")
+            if num_req_missing == 0 and core.hplip_present and core.selected_component == 'hplip' and core.distro_version_supported:
+                path = utils.which('hp-uninstall')
+                ok, choice = tui.enter_choice("HPLIP-%s exists, this may conflict with the new one being installed.\nDo you want to ('i'= Remove and Install, 'o'= Overwrite*, 'q'= Quit)?	:"%(prev_hplip_version),['i','o','q'],'o')
+                if not ok or choice=='q':
+                    log.error("User Exit")
                     sys.exit(0)
+                elif choice == 'i':
+#                    log.info("Uninstalling existing HPLIP-%s"%prev_hplip_version)
+                    sts =core.uninstall(NON_INTERACTIVE_MODE)
 
-                if failed:
-                    log.warn("HPLIP removal failed. The previous install may have been installed using a tarball or this installer.")
-                    log.warn("Continuing to run installer - this installation should overwrite the previous one.")
-
+                    if sts is False: 
+                        log.warn("Failed to uninstall existing HPLIP-%s. This installation will overwrite on existing HPLIP" %prev_hplip_version)
+                    else:
+                        log.debug("HPLIP-%s is uninstalled successfully." %prev_hplip_version)
 
             #
             # POST-DEPEND
             #
-
             tui.title("RUNNING POST-PACKAGE COMMANDS")
             core.run_post_depend(progress_callback)
             log.info("OK")
@@ -714,7 +686,6 @@ def start(language, auto=True, test_depends=False,
             #
             # DEPENDENCIES RE-CHECK
             #
-
             tui.title("RE-CHECKING DEPENDENCIES")
             core.check_dependencies()
 
@@ -762,7 +733,7 @@ def start(language, auto=True, test_depends=False,
                 sys.exit(0)
 
         tui.title("PRE-BUILD COMMANDS")
-        core.run_pre_build(progress_callback)
+        core.run_pre_build(progress_callback, distro_alternate_version)
         log.info("OK")
 
         tui.title("BUILD AND INSTALL")
@@ -792,14 +763,18 @@ def start(language, auto=True, test_depends=False,
         #
 
         tui.title("POST-BUILD COMMANDS")
-        core.run_post_build(progress_callback)
+        core.run_post_build(progress_callback, distro_alternate_version)
+
+        #This call is just to update the cups PPD cache file@ /var/cache/cups/ppds.dat. If this is not called, hp-setup picks incorrect ppd 1st time for some printers. 
+        cups.getSystemPPDs()
 
         #
         # OPEN MDNS MULTICAST PORT
         #
-
+        user_conf = UserConfig()
+        
         if core.selected_options['network']:
-            open_mdns_port = core.get_distro_ver_data('open_mdns_port')
+            open_mdns_port = core.get_distro_ver_data('open_mdns_port',None,distro_alternate_version)
             if open_mdns_port:
                 tui.title("OPEN MDNS/BONJOUR FIREWALL PORT (MULTICAST PORT 5353)")
 
@@ -824,11 +799,8 @@ def start(language, auto=True, test_depends=False,
 
         if current_version >= 0x030902: # 3.9.2
             try:
-                # dBus
-                #import dbus
                 from dbus import SessionBus, lowlevel
             except ImportError:
-                #log.error("Unable to load DBus")
                 pass
             else:
                 try:
@@ -838,10 +810,21 @@ def start(language, auto=True, test_depends=False,
                     tui.title("CLOSE HP_SYSTRAY")
                     log.info("Sending close message to hp-systray (if it is currently running)...")
                     SessionBus().send_message(msg)
+                    time.sleep(0.5)
                 except:
                     pass
+	
+        tui.title("HPLIP UPDATE NOTIFICATION")
+        ok, choice = tui.enter_choice("Do you want to check for HPLIP updates?. (y=yes*, n=no) : ",['y', 'n'], 'y')
+        if not ok or choice != 'y':
+            user_conf.set('upgrade', 'notify_upgrade', 'false')
+        else:
+            user_conf.set('upgrade', 'notify_upgrade', 'true')
+
+        user_conf.set('upgrade','last_upgraded_time',str(int(time.time())))
+        user_conf.set('upgrade','pending_upgrade_time','0')
+
 
-        # Restart or re-plugin if necessary (always True in 2.7.9+)
         if core.selected_component == 'hplip':
             tui.title("RESTART OR RE-PLUG IS REQUIRED")
             cmd = "hp-setup"
@@ -854,23 +837,29 @@ def start(language, auto=True, test_depends=False,
             ok, choice = tui.enter_choice("Restart or re-plug in your printer (r=restart, p=re-plug in*, i=ignore/continue, q=quit) : ",
                 ['r', 'p', 'i'], 'p')
 
-            if not ok: sys.exit(0)
+            if not ok: 
+                start_systray()
+                sys.exit(0)
 
             if choice == 'r':
                 log.note("")
                 log.note("IMPORTANT! Make sure to save all work in all open applications before restarting!")
 
                 ok, ans = tui.enter_yes_no(log.bold("Restart now"), 'n')
-                if not ok: sys.exit(0)
+                if not ok: 
+                    start_systray()
+                    sys.exit(0)
                 if ans:
                     ok = core.restart()
                     if not ok:
                         log.error("Restart failed. Please restart using the system menu.")
 
+                start_systray()
                 sys.exit(0)
 
             elif choice == 'p': # 'p'
                 if not tui.continue_prompt("Please unplug and re-plugin your printer now. "):
+                    start_systray()
                     sys.exit(0)
 
 
@@ -884,13 +873,16 @@ def start(language, auto=True, test_depends=False,
                 install_printer = True
             else:
                 ok, install_printer = tui.enter_yes_no("Would you like to setup a printer now")
-                if not ok: sys.exit(0)
+                if not ok:
+                    start_systray()
+                    sys.exit(0)
 
             if install_printer:
                 log.info("Please make sure your printer is connected and powered on at this time.")
                 if not core.run_hp_setup():
                     log.error("hp-setup failed. Please run hp-setup manually.")
 
+        start_systray()
     except KeyboardInterrupt:
         log.info("")
         log.error("Aborted.")
