@@ -21,6 +21,8 @@
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION 
   WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
+  Author: Naga Samrat Chowdary Narla,
+  Contributor: Sarbeswar Meher
 \*****************************************************************************/
 
 #include "hpmud.h"
@@ -183,6 +185,10 @@ enum HPMUD_RESULT __attribute__ ((visibility ("hidden"))) service_to_channel(mud
    {
       *index = HPMUD_PRINT_CHANNEL;
    }
+   else if (strncasecmp(sn, "hp-ews-ledm", 11) == 0)
+   {
+      *index = HPMUD_EWS_LEDM_CHANNEL;
+   }
    else if (strncasecmp(sn, "hp-ews", 6) == 0)
    {
       *index = HPMUD_EWS_CHANNEL;
@@ -203,6 +209,14 @@ enum HPMUD_RESULT __attribute__ ((visibility ("hidden"))) service_to_channel(mud
    {
       *index = HPMUD_MARVELL_FAX_CHANNEL;
    }
+   else if (strncasecmp(sn, "hp-ledm-scan", 12) == 0)
+   {
+      *index = HPMUD_LEDM_SCAN_CHANNEL;
+   }
+   else if (strncasecmp(sn, "hp-marvell-ews", 11) == 0)
+     {
+       *index = HPMUD_MARVELL_EWS_CHANNEL;
+     }
    /* All the following services require MLC/1284.4. */
    else if (pd->io_mode == HPMUD_RAW_MODE || pd->io_mode == HPMUD_UNI_MODE)
    {
@@ -262,9 +276,9 @@ static int new_device(const char *uri, enum HPMUD_IO_MODE mode, int *result)
 
    if (uri[0] == 0)
       return 0;
-
+   
    pthread_mutex_lock(&msp->mutex);
-
+   
    if (msp->device[i].index)
    {
       BUG("invalid device_open state\n");        /* device is already open for this client, one device per session */
@@ -295,6 +309,7 @@ static int new_device(const char *uri, enum HPMUD_IO_MODE mode, int *result)
    {
       BUG("invalid uri %s\n", uri);
       *result = HPMUD_R_INVALID_URI;
+      index = 0;
       goto bugout;
    }
    msp->device[i].io_mode = mode;
