@@ -2,7 +2,7 @@
 
   hpmud.h - public definitions for multi-point transport driver
 
-  (c) 2004-2008 Copyright Hewlett-Packard Development Company, LP
+  (c) 2004-2015 Copyright Hewlett-Packard Development Company, LP
 
   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to deal
@@ -21,6 +21,7 @@
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
   WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
+  Author: Naga Samrat Chowdary Narla, Yashwant Sahu, Sarbeswar Meher
 \*****************************************************************************/
 
 #ifndef _HPMUD_H
@@ -75,7 +76,17 @@ enum HPMUD_SCANTYPE
    HPMUD_SCANTYPE_SOAP = 3,    /* Wookie (ie:ljcm1017) */
    HPMUD_SCANTYPE_MARVELL = 4,     /* (ie: ljm1005) */
    HPMUD_SCANTYPE_SOAPHT = 5,   /* HorseThief (ie: ljm1522) */
-   HPMUD_SCANTYPE_SCL_DUPLEX = 6
+   HPMUD_SCANTYPE_SCL_DUPLEX = 6,
+   HPMUD_SCANTYPE_LEDM = 7,
+   HPMUD_SCANTYPE_MARVELL2 = 8,     /* (Tsunami lj 1212  and series) */
+};
+
+enum HPMUD_SCANSRC
+{
+   HPMUD_SCANSRC_NA = 0,
+   HPMUD_SCANSRC_FLATBED = 0x1,
+   HPMUD_SCANSRC_ADF= 0x2,
+   HPMUD_SCANSRC_CAMERA = 0x4,
 };
 
 enum HPMUD_STATUSTYPE
@@ -111,13 +122,15 @@ enum HPMUD_PLUGIN_TYPE
 #define HPMUD_S_CONFIG_DOWNLOAD_CHANNEL "HP-CONFIGURATION-DOWNLOAD"
 #define HPMUD_S_MEMORY_CARD_CHANNEL "HP-CARD-ACCESS"
 #define HPMUD_S_EWS_CHANNEL "HP-EWS"
+#define HPMUD_S_EWS_LEDM_CHANNEL "HP-EWS-LEDM"
 #define HPMUD_S_SOAP_SCAN "HP-SOAP-SCAN"
 #define HPMUD_S_SOAP_FAX "HP-SOAP-FAX"
 #define HPMUD_S_DEVMGMT_CHANNEL "HP-DEVMGMT"
 #define HPMUD_S_MARVELL_SCAN_CHANNEL "HP-MARVELL-SCAN"
 #define HPMUD_S_MARVELL_FAX_CHANNEL "HP-MARVELL-FAX"
+#define HPMUD_S_LEDM_SCAN "HP-LEDM-SCAN"
 #define HPMUD_S_WIFI_CHANNEL "HP-WIFICONFIG"
-
+#define HPMUD_S_MARVELL_EWS_CHANNEL "HP-MARVELL-EWS"
 typedef int HPMUD_DEVICE;       /* usb, parallel or jetdirect */
 #define HPMUD_DEVICE_MAX 2      /* zero is not used */
 
@@ -125,7 +138,7 @@ typedef int HPMUD_CHANNEL;
 #define HPMUD_CHANNEL_MAX HPMUD_MAX_CHANNEL_ID
 
 #define HPMUD_LINE_SIZE 256     /* Length of a line. */
-#define HPMUD_BUFFER_SIZE 8192  /* General Read/Write buffer. */
+#define HPMUD_BUFFER_SIZE 16384  /* General Read/Write buffer. */
 
 struct hpmud_dstat
 {
@@ -140,11 +153,12 @@ struct hpmud_model_attributes
 {
    enum HPMUD_IO_MODE prt_mode;        /* print only (io_mode) */
    enum HPMUD_IO_MODE mfp_mode;        /* pml | scan | fax (io_mode) */
-   enum HPMUD_SCANTYPE scantype;       /* 0=none */
+   enum HPMUD_SCANTYPE scantype;       /* scan protocol i.e. SCL, PML, SOAP, MARVELL, LEDM */
    enum HPMUD_STATUSTYPE statustype;
    enum HPMUD_SUPPORT_TYPE support;
    enum HPMUD_PLUGIN_TYPE plugin;
    enum HPMUD_SUPPORT_TYPE reserved[5];
+   enum HPMUD_SCANSRC scansrc; /*Flatbed, ADF, Camera or combination of these*/
 };
 
 #ifdef __cplusplus
