@@ -27,7 +27,7 @@
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
   THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-  Author: Naga Samrat Chowdary Narla,
+  Author: Naga Samrat Chowdary Narla, Sanjay Kumar, Amarnath Chitumalla,Goutam Kodu
 \*****************************************************************************/
 
 #include "HPCupsFilter.h"
@@ -35,7 +35,6 @@
 #include <sys/wait.h>
 #include <sys/utsname.h>
 #include <time.h>
-#include <sys/timeb.h>
 
 #define HP_FILE_VERSION_STR    "03.09.08.0"
 
@@ -295,8 +294,6 @@ DRIVER_ERROR HPCupsFilter::startPage (cups_page_header2_t *cups_header)
     m_JA.color_mode = cups_header->cupsRowStep;
     m_JA.media_source = cups_header->MediaPosition;
 
-    m_JA.color_mode = cups_header->cupsRowStep;
-    m_JA.media_source = cups_header->MediaPosition;
     m_JA.print_borderless = (cups_header->ImagingBoundingBox[0] == 0) ? true : false;
     if (cups_header->Duplex) {
         m_JA.e_duplex_mode  = (cups_header->Tumble == 0) ? DUPLEXMODE_BOOK : DUPLEXMODE_TABLET;
@@ -324,14 +321,8 @@ DRIVER_ERROR HPCupsFilter::startPage (cups_page_header2_t *cups_header)
     int    horz_res = cups_header->HWResolution[0];
     int    vert_res = cups_header->HWResolution[1];
     m_JA.media_attributes.pcl_id = cups_header->cupsInteger[0];
-    m_JA.media_attributes.physical_width   = (cups_header->PageSize[0] * horz_res) / 72;
-    m_JA.media_attributes.physical_height  = (cups_header->PageSize[1] * vert_res) / 72;
-    m_JA.media_attributes.printable_width  = cups_header->cupsWidth;
-    m_JA.media_attributes.printable_height = cups_header->cupsHeight;
-
     m_JA.media_attributes.printable_start_x = (cups_header->Margins[0] * horz_res) / 72;
     m_JA.media_attributes.printable_start_y = ((cups_header->PageSize[1] - cups_header->ImagingBoundingBox[3]) * vert_res) / 72;
-
     m_JA.media_attributes.horizontal_overspray = (xoverspray * horz_res) / 1000;
     m_JA.media_attributes.vertical_overspray   = (yoverspray * vert_res) / 1000;
 
@@ -348,18 +339,26 @@ DRIVER_ERROR HPCupsFilter::startPage (cups_page_header2_t *cups_header)
     }
 
 //  Get printer platform name
-    if (((attr = ppdFindAttr(m_ppd, "hpPrinterPlatform", NULL)) != NULL) &&
-        (attr->value != NULL)) {
-        strncpy(m_JA.printer_platform, attr->value, sizeof(m_JA.printer_platform)-1);
-        if (m_iLogLevel & BASIC_LOG) {
-            dbglog("HPCUPS: found Printer Platform, it is - %s", attr->value);
-        }
-      if(strcmp(m_JA.printer_platform, "ljzjscolor") == 0)
-      {
-        if(((attr = ppdFindAttr(m_ppd, "hpLJZjsColorVersion", NULL)) != NULL) && (attr->value != NULL))
-          m_JA.printer_platform_version = atoi(attr->value);
-      }
+    if (((attr = ppdFindAttr(m_ppd, "hpPrinterPlatform", NULL)) != NULL) && (attr->value != NULL)) {
+    
+		strncpy(m_JA.printer_platform, attr->value, sizeof(m_JA.printer_platform)-1);
+
+		if (m_iLogLevel & BASIC_LOG) {
+			dbglog("HPCUPS: found Printer Platform, it is - %s\n", attr->value);
+		}        
+        
+		if(strcmp(m_JA.printer_platform, "ljzjscolor") == 0){
+			if(((attr = ppdFindAttr(m_ppd, "hpLJZjsColorVersion", NULL)) != NULL) && (attr->value != NULL)){
+				m_JA.printer_platform_version = atoi(attr->value);
+			}
+		}   
     }
+    
+//Get Raster Preprocessing status  
+	if(((attr = ppdFindAttr(m_ppd, "hpReverseRasterPages", NULL)) != NULL) && (attr->value != NULL)){				
+		m_JA.pre_process_raster = atoi(attr->value);
+	}      
+    
 
 // Get the encapsulation technology from ppd
 
@@ -371,8 +370,23 @@ DRIVER_ERROR HPCupsFilter::startPage (cups_page_header2_t *cups_header)
             return SYSTEM_ERROR;
     }
     strncpy(m_JA.printer_language, attr->value, sizeof(m_JA.printer_language)-1);
+
+    if(strcmp(m_JA.printer_language, "hbpl1") == 0) {
+        m_JA.media_attributes.physical_width = cups_header->PageSize[0];
+        m_JA.media_attributes.physical_height = cups_header->PageSize[1];
+        m_JA.media_attributes.printable_width = ((cups_header->ImagingBoundingBox[2]-cups_header->ImagingBoundingBox[0]) * horz_res) / 72;
+        m_JA.media_attributes.printable_height = ((cups_header->ImagingBoundingBox[3]-cups_header->ImagingBoundingBox[1]) * vert_res) / 72;
+        strncpy(m_JA.media_attributes.PageSizeName, cups_header->cupsPageSizeName, sizeof(m_JA.media_attributes.PageSizeName)-1);
+    }
+    else{
+        m_JA.media_attributes.physical_width   = (cups_header->PageSize[0] * horz_res) / 72;
+        m_JA.media_attributes.physical_height  = (cups_header->PageSize[1] * vert_res) / 72;
+        m_JA.media_attributes.printable_width  = cups_header->cupsWidth;
+        m_JA.media_attributes.printable_height = cups_header->cupsHeight;
+    }
+
     if (m_iLogLevel & BASIC_LOG) {
-        dbglog("HPCUPS: found Printer Language, it is - %s", attr->value);
+        dbglog("HPCUPS: found Printer Language, it is - %s\n", attr->value);
     }
 
 //  Fill in the other PCL header info
@@ -400,6 +414,10 @@ DRIVER_ERROR HPCupsFilter::startPage (cups_page_header2_t *cups_header)
             m_JA.printer_name[i++] = *ptr++;
         }
     }
+
+	string strPrinterURI="" ,strPrinterName= "";	
+    m_DBusComm.initDBusComm(DBUS_PATH,DBUS_INTERFACE, getenv("DEVICE_URI"), m_JA.printer_name);
+	
     ptr = strstr(m_argv[5], "job-uuid");
     if (ptr) {
         strncpy(m_JA.uuid, ptr + strlen("job-uuid=urn:uuid:"), sizeof(m_JA.uuid)-1);
@@ -416,7 +434,12 @@ DRIVER_ERROR HPCupsFilter::startPage (cups_page_header2_t *cups_header)
     if ((err = m_Job.Init(m_pSys, &m_JA, encap_interface)) != NO_ERROR)
     {
         if (err == PLUGIN_LIBRARY_MISSING)
+        {
             fputs ("STATE: +hplip.plugin-error\n", stderr);
+
+            m_DBusComm.sendEvent(EVENT_PRINT_FAILED_MISSING_PLUGIN, "Plugin missing", m_JA.job_id, m_JA.user_name);
+
+        }
         dbglog ("m_Job initialization failed with error = %d", err);
         ppdClose(m_ppd);
         m_ppd = NULL;
@@ -424,10 +447,11 @@ DRIVER_ERROR HPCupsFilter::startPage (cups_page_header2_t *cups_header)
     }
 
     if (m_iLogLevel & BASIC_LOG) {
-        dbglog("HPCUPS: returning NO_ERROR from startPage");
+        dbglog("HPCUPS: returning NO_ERROR from startPage\n");
     }
 
     m_pPrinterBuffer = new BYTE[cups_header->cupsWidth * 4 + 32];
+    
 
     return NO_ERROR;
 }
@@ -440,13 +464,13 @@ int HPCupsFilter::StartPrintJob(int  argc, char *argv[])
 
     memset(&m_JA, 0, sizeof(JobAttributes));
     struct    tm       *t;
-    struct    timeb    tb;
+    struct timeval	 tv;
     time_t             long_time;
     time(&long_time);
     t = localtime(&long_time);
-    ftime(&tb);
+    gettimeofday(&tv, NULL);
     strncpy(m_JA.job_start_time, asctime(t), sizeof(m_JA.job_start_time)-1);    // returns Fri Jun  5 08:12:16 2009
-    snprintf(m_JA.job_start_time+19, sizeof(m_JA.job_start_time) - 20, ":%d %d", tb.millitm, t->tm_year + 1900); // add milliseconds
+    snprintf(m_JA.job_start_time+19, sizeof(m_JA.job_start_time) - 20, ":%ld %d", tv.tv_usec/1000, t->tm_year + 1900); // add milliseconds
 
     getLogLevel();
     m_JA.job_id = atoi(argv[1]);
@@ -538,18 +562,29 @@ int HPCupsFilter::StartPrintJob(int  argc, char *argv[])
     return 0;
 }
 
-bool HPCupsFilter::isBlankRaster(BYTE *input_raster, int length_in_bytes)
-{
+bool HPCupsFilter::isBlankRaster(BYTE *input_raster, cups_page_header2_t *header)
+{   
+    int length_in_bytes = (int)header->cupsBytesPerLine;
     if (input_raster == NULL) {
         return true;
     }
-    if (*input_raster == 0xFF &&
-        !(memcmp(input_raster + 1, input_raster, length_in_bytes - 1))) {
-    return true;
+
+    if(header->cupsColorSpace == CUPS_CSPACE_K){
+	if (*input_raster == 0x00 &&
+            !(memcmp(input_raster + 1, input_raster, length_in_bytes - 1))) {
+        return true;
+        }
+    }
+    else{
+        if (*input_raster == 0xFF &&
+              !(memcmp(input_raster + 1, input_raster, length_in_bytes - 1))) {
+        return true;
+        }
     }
     return false;
 }
 
+
 int HPCupsFilter::processRasterData(cups_raster_t *cups_raster)
 {
     FILE                   *kfp = NULL;
@@ -561,14 +596,32 @@ int HPCupsFilter::processRasterData(cups_raster_t *cups_raster)
     DRIVER_ERROR           err;
     int                    ret_status = 0;
 
+    char hpPreProcessedRasterFile[64]; //temp file needed to store raster data with swaped pages.
+
+    strcpy(hpPreProcessedRasterFile, "/var/log/hp/tmp/hplipSwapedPagesXXXXXX");
+       
+
     while (cupsRasterReadHeader2(cups_raster, &cups_header))
     {
         current_page_number++;
 
         if (current_page_number == 1) {
+            
             if (startPage(&cups_header) != NO_ERROR) {
                 return JOB_CANCELED;
             }
+            
+            if(m_JA.pre_process_raster) {
+		        err = m_Job.preProcessRasterData(&cups_raster, &cups_header, hpPreProcessedRasterFile);
+				if (err != NO_ERROR) {
+					if (m_iLogLevel & BASIC_LOG) {
+						dbglog ("DEBUG: Job::StartPage failed with err = %d\n", err);
+					}
+					ret_status = JOB_CANCELED;
+					break;
+				}   
+			}
+            
             if (cups_header.cupsColorSpace == CUPS_CSPACE_RGBW) {
                 rgbRaster = new BYTE[cups_header.cupsWidth * 3];
                 if (rgbRaster == NULL) {
@@ -604,12 +657,13 @@ int HPCupsFilter::processRasterData(cups_raster_t *cups_raster)
             ret_status = JOB_CANCELED;
             break;
         }
-
+        
         if (m_iLogLevel & SAVE_INPUT_RASTERS)
         {
-            char    szFileName[32];
+            char    szFileName[64];
             memset(szFileName, 0, sizeof(szFileName));
-            snprintf (szFileName, sizeof(szFileName), "/tmp/hpcupsfilterc_%d.bmp", current_page_number);
+            snprintf (szFileName, sizeof(szFileName), "/var/log/hp/tmp/hpcupsfilterc_%d.bmp", current_page_number);
+
             if (cups_header.cupsColorSpace == CUPS_CSPACE_RGBW ||
                 cups_header.cupsColorSpace == CUPS_CSPACE_RGB)
             {
@@ -623,18 +677,30 @@ int HPCupsFilter::processRasterData(cups_raster_t *cups_raster)
                 kfp = fopen (szFileName, "w");
                 chmod (szFileName, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
             }
+
             WriteBMPHeader (cfp, cups_header.cupsWidth, cups_header.cupsHeight, COLOR_RASTER);
             WriteBMPHeader (kfp, cups_header.cupsWidth, cups_header.cupsHeight, BLACK_RASTER);
         }
 
-        fprintf(stderr, "PAGE: %d %s", current_page_number, m_argv[4]);
+        fprintf(stderr, "PAGE: %d %s\r\n", current_page_number, m_argv[4]);
         // Iterating through the raster per page
         for (int y = 0; y < (int) cups_header.cupsHeight; y++) {
             cupsRasterReadPixels (cups_raster, m_pPrinterBuffer, cups_header.cupsBytesPerLine);
             color_raster = rgbRaster;
             black_raster = kRaster;
 
-            if (this->isBlankRaster((BYTE *) m_pPrinterBuffer, (int) cups_header.cupsBytesPerLine)) {
+            if(y == 0 && (0 == strcmp(m_JA.printer_language, "ljmono")) )
+			{
+				//For ljmono, make sure that first line is not a blankRaster line.Otherwise printer 
+				//may not skip blank lines before actual data
+				//Need to revisit to crosscheck if it is a firmware issue.
+				
+				*m_pPrinterBuffer = 0x01;  
+				dbglog("First raster data plane.." );
+			}
+			
+            if (this->isBlankRaster((BYTE *) m_pPrinterBuffer, &cups_header)) {
+            
                 color_raster = NULL;
                 black_raster = NULL;
             }
@@ -658,6 +724,8 @@ int HPCupsFilter::processRasterData(cups_raster_t *cups_raster)
         delete [] kRaster;
         delete [] rgbRaster;
     }
+
+    unlink(hpPreProcessedRasterFile); 
     return ret_status;
 }
 
@@ -680,7 +748,7 @@ static BYTE pixel_value[8] = {
         int    k = 0;
         BYTE   *pIn = m_pPrinterBuffer;
         BYTE   kVal = 0;
-        BYTE   b;
+        BYTE   white=0;
         BYTE   *rgb = rgbRaster;
         BYTE   *black   = kRaster;
         memset (kRaster, 0, cups_header->cupsWidth);
@@ -690,24 +758,34 @@ static BYTE pixel_value[8] = {
             rgb[0] = *pIn++;
             rgb[1] = *pIn++;
             rgb[2] = *pIn++;
-            b = *pIn++;
+            white = *pIn++;
 
-            if (b != 0 && b != 0xFF) {
-
-#ifdef __linux
-                rgb[0] -= (255 - b);
-                rgb[1] -= (255 - b);
-                rgb[2] -= (255 - b);
-                
-#else  // This alternate path is for Mac....
-
-                rgb[0] &= b;
-                rgb[1] &= b;
-                rgb[2] &= b;
-#endif
+            if(white == 0)
+            {
+            	//If W component is 0 (means black is 1) then no need of having RGB for that pixel.
+            	//ghostscript >= 8.71 sends both W and RGB for black pixel(i.e RGBW=(0,0,0,0)).
+				kVal |= pixel_value[k];
+				rgb[0] = 0xFF;
+				rgb[1] = 0xFF;
+				rgb[2] = 0xFF;
+            }
+            else if(white == 0xFF)
+            {
+            	kVal |= 0;
             }
             else
-                kVal |= (b == 0) ? pixel_value[k] : 0;
+            {
+              int cr,cg,cb;
+              cr = rgb[0] - (int)(255 - white);
+              rgb[0] = cr >= 0 ? cr : 0;
+
+              cg = rgb[1] - (int)(255 - white);
+              rgb[1] = cg >= 0 ? cg : 0;
+
+              cb = rgb[2] - (int)(255 - white);
+              rgb[2] = cb >= 0 ? cb : 0;
+            }
+
             rgb += 3;
             if (k == 7) {
                 *black++ = kVal;
