@@ -65,7 +65,7 @@ uint32_t (*convert_endian_l)(uint32_t);
 uint16_t (*convert_endian_s)(uint16_t);
 
 static int iLogLevel = 1;
-char hpFileName[] = "/tmp/hplipfaxXXXXXX";
+char hpFileName[64] ;
 
 #define TIFF_HDR_SIZE 8
 #define LITTLE_ENDIAN_MODE I
@@ -439,14 +439,22 @@ int ProcessTiffData(int fromFD, int toFD)
     int bytes_written = 0;
     int ret_status = 0;
     int bytes_read = 0;
-    char hpTiffFileName[] = "/tmp/hpliptiffXXXXXX";
+    char hpTiffFileName[64];
     long input_file_size = 0;
+    snprintf(hpTiffFileName,sizeof(hpTiffFileName), "%s/hpliptiffXXXXXX","/var/log/hp/tmp");
+
 
     fdTiff = mkstemp (hpTiffFileName);
     if (fdTiff < 0)
     {
-        BUG ("ERROR: Unable to open Fax output file - %s for writing\n", hpTiffFileName);
-        return 1;
+//        DBG ("Warning: Unable to open Fax output file - %s for writing, so trying another location\n", hpTiffFileName);
+//        strncpy(hpTiffFileName,"/var/log/hp/hpliptiffXXXXXX",sizeof(hpTiffFileName));
+//        fdTiff = mkstemp (hpTiffFileName);
+//        if (fdTiff < 0)
+        {
+            BUG("ERROR: Unable to open Fax output file - %s for writing\n", hpTiffFileName);
+            return 1;
+        }
     }
 
     memset (szFileHeader, 0, sizeof (szFileHeader));
@@ -593,6 +601,10 @@ int ProcessTiffData(int fromFD, int toFD)
     HPLIPPUTINT32 ((szFileHeader + 9), page_counter);
     write (toFD, szFileHeader + 9, 4);
 
+	if (!(iLogLevel & SAVE_PCL_FILE))
+	{
+         unlink(hpTiffFileName);
+	}
     return ret_status;
 }
 
@@ -602,7 +614,6 @@ int send_data_to_stdout(int fromFD)
     int     iSize, i;
     int     len;
     BYTE    *pTmp = NULL;
-    FILE    *fp = NULL;
 
     iSize = lseek (fromFD, 0, SEEK_END);
     lseek (fromFD, 0, SEEK_SET);
@@ -622,28 +633,12 @@ int send_data_to_stdout(int fromFD)
         }
     }
 
-    fp = NULL;
-    if (iLogLevel & SAVE_PCL_FILE)
-    {
-        fp = fopen ("/tmp/hpcupsfax.out", "w");
-        system ("chmod 666 /tmp/hpcupsfax.out");
-    }
-    
     while ((len = read (fromFD, pTmp, iSize)) > 0)
     {
         write (STDOUT_FILENO, pTmp, len);
-        if (iLogLevel & SAVE_PCL_FILE && fp)
-        {
-            fwrite (pTmp, 1, len, fp);
-        }
     }
     free (pTmp);
 
-    if (fp)
-    {
-        fclose (fp);
-    }
-
     return 0;
 }
 
@@ -684,11 +679,19 @@ int main (int argc, char **argv)
          i++;
     }
 
+    snprintf(hpFileName,sizeof(hpFileName),"%s/hplipfaxLog_XXXXXX","/var/log/hp/tmp");
+
     fdFax = mkstemp (hpFileName);
     if (fdFax < 0)
     {
-        BUG ("ERROR: Unable to open Fax output file - %s for writing\n", hpFileName);
-        return 1;
+//        DBG ("Warning: Unable to open Fax output file - %s for writing,so trying another location.\n", hpFileName);
+//        strncpy(hpFileName,"/var/log/hp/hplipfaxLog_XXXXXX",sizeof(hpFileName));
+//        fdFax = mkstemp (hpFileName);
+//        if (fdFax < 0)
+        {
+            BUG ("ERROR: Unable to open Fax output file - %s for writing\n", hpFileName);
+            return 1;
+        }
     }
 
     /*********** MAIN ***********/
@@ -780,9 +783,15 @@ EPILOGUE:
     {
         close (fd);
     }
+
     if (fdFax > 0)
     {
         close (fdFax);
+	if (!(iLogLevel & SAVE_PCL_FILE))
+	{
+             //Retain the intermediate file only if it is needed for debugging purpose.
+             unlink(hpFileName);
+	}
     }
 
     return status;
