@@ -21,6 +21,7 @@
   WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
   Primary Author: Naga Samrat Chowary, Narla 
+  Contributing Authors: Yashwant Kumar Sahu,Sarbeswar Meher
 
 *************************************************************************************/
 
@@ -90,6 +91,8 @@ static struct ledm_session *create_session()
   ps->tag = "LEDM";
   ps->dd = -1;
   ps->cd = -1;
+  ps->job_id = 0;
+  ps->page_id = 0;
   return ps;
 }
 
@@ -371,9 +374,8 @@ SANE_Status __attribute__ ((visibility ("hidden"))) ledm_open(SANE_String_Const
 
   if (hpmud_open_device(session->uri, ma.mfp_mode, &session->dd) != HPMUD_R_OK)
   {
-    free(session);
-    session = NULL;
-    return SANE_STATUS_IO_ERROR;
+    stat = SANE_STATUS_IO_ERROR;
+    goto bugout;
   }
 
   init_options(session);
@@ -381,6 +383,7 @@ SANE_Status __attribute__ ((visibility ("hidden"))) ledm_open(SANE_String_Const
   if (bb_open(session))
   {
     stat = SANE_STATUS_IO_ERROR;
+    goto bugout;
   }
 
   /* Set supported Scan Modes as determined by bb_open. */
@@ -411,7 +414,22 @@ SANE_Status __attribute__ ((visibility ("hidden"))) ledm_open(SANE_String_Const
 
   stat = SANE_STATUS_GOOD;
 
-return stat;
+bugout:
+
+   if (stat != SANE_STATUS_GOOD)
+   {
+      if (session)
+      {
+         if (session->cd > 0)
+            hpmud_close_channel(session->dd, session->cd);
+         if (session->dd > 0)
+            hpmud_close_device(session->dd);
+         free(session);
+         session = NULL;
+      }
+   }
+
+   return stat;
 }
 
 const SANE_Option_Descriptor *ledm_get_option_descriptor(SANE_Handle handle, SANE_Int option)
@@ -506,11 +524,20 @@ SANE_Status ledm_control_option(SANE_Handle handle, SANE_Int option, SANE_Action
                  i = session->adf_resolutionList[0] + 1;
                  while(i--) session->resolutionList[i] = session->adf_resolutionList[i];
                }
-               mset_result |= SANE_INFO_RELOAD_PARAMS | SANE_INFO_RELOAD_OPTIONS;
-               stat = SANE_STATUS_GOOD;
+               ps->currentResolution = session->resolutionList[1];
                break;
              }
            }
+           /*For some devices resolution varies, when we change 'source' in Xsane.
+           Hence need to update the resolution  */
+           if (i>1) /*Number of sources > 1*/
+           {
+             if(session->platen_resolutionList[1] != session->adf_resolutionList[1])
+                 ps->currentResolution = session->resolutionList[1];
+           }
+           mset_result |= SANE_INFO_RELOAD_PARAMS | SANE_INFO_RELOAD_OPTIONS;
+           stat = SANE_STATUS_GOOD;
+           break;
          }
          else
          {  /* Set default. */
@@ -533,12 +560,17 @@ SANE_Status ledm_control_option(SANE_Handle handle, SANE_Int option, SANE_Action
                if (ps->resolutionList[i] == *int_value)
                {
                   ps->currentResolution = *int_value;
-	          if(ps->currentResolution == 4800) SendScanEvent(ps->uri, EVENT_SIZE_WARNING);
+                  if(ps->currentResolution == 4800) SendScanEvent(ps->uri, EVENT_SIZE_WARNING);
                   mset_result |= SANE_INFO_RELOAD_PARAMS;
                   stat = SANE_STATUS_GOOD;
                   break;
                }
             }
+            if (stat != SANE_STATUS_GOOD)
+            {
+                ps->currentResolution = ps->resolutionList[1];
+                stat = SANE_STATUS_GOOD;
+            }
          }
          else
          {  /* Set default. */
@@ -557,9 +589,13 @@ SANE_Status ledm_control_option(SANE_Handle handle, SANE_Int option, SANE_Action
             if (*int_value >= LEDM_CONTRAST_MIN && *int_value <= LEDM_CONTRAST_MAX)
             {
                ps->currentContrast = *int_value;
-               stat = SANE_STATUS_GOOD;
-               break;
             }
+            else
+            {
+                ps->currentContrast = LEDM_CONTRAST_DEFAULT;
+            }
+            mset_result |= SANE_INFO_RELOAD_PARAMS;
+            stat = SANE_STATUS_GOOD;
          }
          else
          {  /* Set default. */
@@ -759,12 +795,13 @@ SANE_Status ledm_start(SANE_Handle handle)
   }   
 
   /* If input is ADF and ADF is empty, return SANE_STATUS_NO_DOCS. */
-  if (ps->currentInputSource==IS_ADF)
+  if (ps->currentInputSource==IS_ADF || ps->currentInputSource ==IS_ADF_DUPLEX)
   {
     ret = bb_is_paper_in_adf(ps);   /* 0 = no paper in adf, 1 = paper in adf, -1 = error */
     if (ret == 0)
     {
       stat = SANE_STATUS_NO_DOCS;   /* done scanning */
+      SendScanEvent (ps->uri, EVENT_SCAN_ADF_NO_DOCS);
       goto bugout;
     }
     else if (ret < 0)
@@ -775,12 +812,16 @@ SANE_Status ledm_start(SANE_Handle handle)
   }
 
    /* Start scan and get actual image traits. */
-  if (bb_start_scan(ps))
+  stat = bb_start_scan(ps);
+  if (stat != SANE_STATUS_GOOD)
+    goto bugout;
+
+  if(ps->user_cancel)
   {
-    stat = SANE_STATUS_IO_ERROR;
+    stat = SANE_STATUS_GOOD ;
     goto bugout;
   }
-
+   
   SendScanEvent(ps->uri, EVENT_START_SCAN_JOB);
 
   memset(xforms, 0, sizeof(xforms));
@@ -973,7 +1014,6 @@ void ledm_cancel(SANE_Handle handle)
 
 void ledm_close(SANE_Handle handle)
 {
-  struct hpmud_model_attributes ma;
   struct ledm_session *ps = (struct ledm_session *)handle;
 
   if (ps == NULL || ps != session)
