@@ -27,6 +27,7 @@
   hardware works. The Windows driver has the same limitation.
 
   Author: David Suffield
+  Contributor: Sarbeswar Meher
 
 \************************************************************************************/
 
@@ -513,16 +514,7 @@ SANE_Status soapht_open(SANE_String_Const device, SANE_Handle *handle)
    soapht_control_option(session, SOAP_OPTION_INPUT_SOURCE, SANE_ACTION_SET_AUTO, NULL, NULL); /* set default option */  
 
    /* Set supported resolutions. */
-   i=1;
-   session->resolutionList[i++] = 75;
-   session->resolutionList[i++] = 100;
-   session->resolutionList[i++] = 150;
-   session->resolutionList[i++] = 200;
-   session->resolutionList[i++] = 300;
-   session->resolutionList[i++] = 600;
-   session->resolutionList[i++] = 1200;
-   session->resolutionList[0] = i-1;    /* length of word_list */
-   soapht_control_option(session, SOAP_OPTION_SCAN_RESOLUTION, SANE_ACTION_SET_AUTO, NULL, NULL); /* set default option */
+     soapht_control_option(session, SOAP_OPTION_SCAN_RESOLUTION, SANE_ACTION_SET_AUTO, NULL, NULL); /* set default option */
 
    /* Set supported contrast. */
    soapht_control_option(session, SOAP_OPTION_CONTRAST, SANE_ACTION_SET_AUTO, NULL, NULL); /* set default option */
@@ -659,21 +651,33 @@ SANE_Status soapht_control_option(SANE_Handle handle, SANE_Int option, SANE_Acti
          }
          else if (action == SANE_ACTION_SET_VALUE)
          {
-            for (i=0; ps->inputSourceList[i]; i++)
-            {
-               if (strcasecmp(ps->inputSourceList[i], value) == 0)
+          for (i=0; ps->inputSourceList[i]; i++)
+           {
+             if (strcasecmp(ps->inputSourceList[i], value) == 0)
+             {
+               ps->currentInputSource = ps->inputSourceMap[i];
+               set_input_source_side_effects(ps, ps->currentInputSource);
+               if(ps->currentInputSource == IS_ADF || ps->currentInputSource == IS_ADF_DUPLEX)
                {
-                  ps->currentInputSource = ps->inputSourceMap[i];
-                  set_input_source_side_effects(ps, ps->currentInputSource);
-                  stat = SANE_STATUS_GOOD;
-                  break;
+                 i = ps->adf_resolutionList[0] + 1;
+                 while(i--) ps->resolutionList[i] = ps->adf_resolutionList[i];
                }
-            }
+               else //if(ps->currentInputSource == IS_PLATEN) 
+               {
+                 i = ps->platen_resolutionList[0] + 1;
+                 while(i--) ps->resolutionList[i] = ps->platen_resolutionList[i];
+               }
+               mset_result |= SANE_INFO_RELOAD_PARAMS | SANE_INFO_RELOAD_OPTIONS;
+               stat = SANE_STATUS_GOOD;
+               break;
+             }
+           }
          }
          else
          {  /* Set default. */
             ps->currentInputSource = IS_PLATEN;
             set_input_source_side_effects(ps, ps->currentInputSource);
+            mset_result |= SANE_INFO_RELOAD_PARAMS | SANE_INFO_RELOAD_OPTIONS;
             stat = SANE_STATUS_GOOD;
          }
          break;
@@ -695,6 +699,11 @@ SANE_Status soapht_control_option(SANE_Handle handle, SANE_Int option, SANE_Acti
                   break;
                }
             }
+            if (stat != SANE_STATUS_GOOD)
+            {
+                ps->currentResolution = ps->resolutionList[1];
+                stat = SANE_STATUS_GOOD;
+            }
          }
          else
          {  /* Set default. */
@@ -713,9 +722,13 @@ SANE_Status soapht_control_option(SANE_Handle handle, SANE_Int option, SANE_Acti
             if (*int_value >= SOAP_CONTRAST_MIN && *int_value <= SOAP_CONTRAST_MAX)
             {
                ps->currentContrast = *int_value;
-               stat = SANE_STATUS_GOOD;
-               break;
             }
+            else
+            {
+               ps->currentContrast = SOAP_CONTRAST_DEFAULT;
+            }
+            mset_result |= SANE_INFO_RELOAD_PARAMS;
+            stat = SANE_STATUS_GOOD;
          }
          else
          {  /* Set default. */
@@ -908,7 +921,11 @@ SANE_Status soapht_start(SANE_Handle handle)
    int stat, ret;
 
    DBG8("sane_hpaio_start()\n");
-
+   
+    ps -> user_cancel = 0;
+    ps -> cnt = 0;
+    ps -> index = 0;
+  
    if (set_extents(ps))
    {
       BUG("invalid extents: tlx=%d brx=%d tly=%d bry=%d minwidth=%d minheight%d maxwidth=%d maxheight=%d\n",
@@ -924,6 +941,7 @@ SANE_Status soapht_start(SANE_Handle handle)
       if (ret == 0)
       {
          stat = SANE_STATUS_NO_DOCS;     /* done scanning */
+         SendScanEvent (ps->uri, EVENT_SCAN_ADF_NO_DOCS);
          goto bugout;
       }
       else if (ret < 0)
@@ -939,7 +957,7 @@ SANE_Status soapht_start(SANE_Handle handle)
       stat = SANE_STATUS_IO_ERROR;
       goto bugout;
    }
-
+   SendScanEvent(ps->uri, EVENT_START_SCAN_JOB);
    memset(xforms, 0, sizeof(xforms));    
 
    /* Setup image-processing pipeline for xform. */
@@ -1077,7 +1095,13 @@ SANE_Status soapht_read(SANE_Handle handle, SANE_Byte *data, SANE_Int maxLength,
    int ret, stat=SANE_STATUS_IO_ERROR;
 
    DBG8("sane_hpaio_read() handle=%p data=%p maxLength=%d\n", (void *)handle, data, maxLength);
-
+   if(ps->user_cancel)
+   {
+     DBG8("soapht_read() EVENT_SCAN_CANCEL****uri=%s\n", ps->uri);
+     SendScanEvent(ps->uri, EVENT_SCAN_CANCEL);
+     return SANE_STATUS_CANCELLED;
+   }
+  
    ret = get_ip_data(ps, data, maxLength, length);
 
    if(ret & (IP_INPUT_ERROR | IP_FATAL_ERROR))
@@ -1087,7 +1111,10 @@ SANE_Status soapht_read(SANE_Handle handle, SANE_Byte *data, SANE_Int maxLength,
    }
 
    if (ret & IP_DONE)
+   {
       stat = SANE_STATUS_EOF;
+      SendScanEvent(ps->uri, EVENT_END_SCAN_JOB);
+   }
    else
       stat = SANE_STATUS_GOOD;
 
@@ -1118,7 +1145,7 @@ void soapht_cancel(SANE_Handle handle)
     * Sane_cancel is always called at the end of the scan job. Note that on a multiple page scan job 
     * sane_cancel is called only once.
     */
-
+   ps -> user_cancel = 1;
    if (ps->ip_handle)
    {
       ipClose(ps->ip_handle); 
@@ -1127,7 +1154,3 @@ void soapht_cancel(SANE_Handle handle)
    ps->bb_end_scan(ps, 0);
 } /* soapht_cancel */
 
-
-
-
-
