@@ -233,16 +233,16 @@ class SetupForm(SetupForm_base):
 
             norm_model = models.normalizeModelName(model).lower()
 
-            core = core_install.CoreInstall()
+            core = core_install.CoreInstall(core_install.MODE_CHECK)
             core.set_plugin_version()
             plugin = self.mq.get('plugin', PLUGIN_NONE)
             plugin_reason = self.mq.get('plugin-reason', PLUGIN_REASON_NONE)
-            if plugin > PLUGIN_NONE and not core.check_for_plugin():
+            if plugin > PLUGIN_NONE and core.check_for_plugin() != PLUGIN_INSTALLED:
                 ok, sudo_ok = pkit.run_plugin_command(plugin == PLUGIN_REQUIRED, plugin_reason)
                 if not sudo_ok:
                     self.FailureUI(self.__tr("<b>Unable to find an appropriate su/sudo utility to run hp-plugin.</b><p>Install kdesu, gnomesu, or gksu.</p>"))
                     return
-                if not ok or not core.check_for_plugin():
+                if not ok or core.check_for_plugin() != PLUGIN_INSTALLED:
                     if plugin == PLUGIN_REQUIRED:
                         self.FailureUI(self.__tr("<b>The printer you are trying to setup requires a binary driver plug-in and it failed to install.</b><p>Please check your internet connection and try again.</p><p>Visit <u>http://hplipopensource.com</u> for more information.</p>"))
                         return
@@ -536,33 +536,13 @@ class SetupForm(SetupForm_base):
         if ppds is None or not ppds:
             ppds = cups.getSystemPPDs()
 
-        #print ppds
-
-        default_model = utils.xstrip(model.replace('series', '').replace('Series', ''), '_')
-        stripped_model = cups.stripModel2(models.normalizeModelName(model).lower())
-        
-        
-        #Check if common ppd name is already given in models.dat(This is needed because in case of devices having more than one derivatives
-        #will have diffrent model name strings in device ID, because of which we don't get the common ppd name for search)
-       
-        ppd_name = self.mq.get('ppd-name',0)
-        
-        if ppd_name == 0:
-        	self.ppd = cups.getPPDFile2(stripped_model, ppds)
-        else:
-            self.ppd = cups.getPPDFile2(ppd_name, ppds)
-        
+        self.ppd = cups.getPPDFile2(self.mq,  model , ppds)
         log.debug(self.ppd)
         self.ppdListView.clear()
 
         if self.ppd is not None:
-            #for ppd in self.ppd_dict:
             PPDListViewItem(self.ppdListView, self.ppd[0], self.ppd[1])
 
-#            i = self.ppdListView.firstChild()
-#            self.ppdListView.setCurrentItem(i)
-#            self.ppdListView.setSelected(i, True)
-#            self.ppd_file = self.ppdListView.currentItem().ppd_file
             self.ppd_file = self.ppd[0]
             log.debug(self.ppd_file)
 
@@ -606,13 +586,14 @@ class SetupForm(SetupForm_base):
 
         printer_name = default_model
 
+        installed_printer_names = device.getSupportedCUPSPrinterNames(['hp'])
         # Check for duplicate names
-        if self.device_uri in self.installed_print_devices and \
-            printer_name in self.installed_print_devices[self.device_uri]:
+        if (self.device_uri in self.installed_print_devices and printer_name in self.installed_print_devices[self.device_uri]) \
+           or (printer_name in installed_printer_names):
                 i = 2
                 while True:
                     t = printer_name + "_%d" % i
-                    if t not in self.installed_print_devices[self.device_uri]:
+                    if (t not in installed_printer_names) and (self.device_uri not in self.installed_print_devices or t not in self.installed_print_devices[self.device_uri]):
                         printer_name += "_%d" % i
                         break
                     i += 1
@@ -713,15 +694,16 @@ class SetupForm(SetupForm_base):
         default_model = utils.xstrip(model.replace('series', '').replace('Series', ''), '_')
 
         fax_name = default_model + "_fax"
+        installed_fax_names = device.getSupportedCUPSPrinterNames(['hpfax'])
 
         # Check for duplicate names
-        if self.fax_uri in self.installed_fax_devices and \
-            fax_name in self.installed_fax_devices[self.fax_uri]:
+        if (self.fax_uri in self.installed_fax_devices and fax_name in self.installed_fax_devices[self.fax_uri]) \
+           or (fax_name in installed_fax_names):
         #if fax_name in self.installed_queues or fax_name == self.printer_name:
                 i = 2
                 while True:
                     t = fax_name + "_%d" % i
-                    if t not in self.installed_fax_devices[self.fax_uri]:
+                    if (t not in installed_fax_names) and (self.fax_uri not in self.installed_fax_devices or t not in self.installed_fax_devices[self.fax_uri]):
                         fax_name += "_%d" % i
                         break
                     i += 1
@@ -886,43 +868,19 @@ class SetupForm(SetupForm_base):
 
             self.FailureUI(self.__tr("<b>Printer queue setup failed.</b><p>Please restart CUPS and try again."))
         else:
-            # TODO:
-            #service.sendEvent(self.hpssd_sock, EVENT_CUPS_QUEUES_CHANGED, device_uri=self.device_uri)
-            pass
+            # sending Event to add this device in hp-systray
+            utils.sendEvent(EVENT_CUPS_QUEUES_CHANGED,self.device_uri, self.printer_name)
 
         QApplication.restoreOverrideCursor()
 
     def setupFax(self):
         QApplication.setOverrideCursor(QApplication.waitCursor)
-
-        if self.mq.get('fax-type', FAX_TYPE_NONE) == FAX_TYPE_MARVELL:
-            fax_ppd_name = "HP-Fax3-hplip" # Fixed width (2528 pixels) and 300dpi rendering
-            nick = "HP Fax 3"
-        if self.mq.get('fax-type', FAX_TYPE_NONE) == FAX_TYPE_SOAP or self.mq.get('fax-type', FAX_TYPE_NONE) == FAX_TYPE_LEDMSOAP:
-            fax_ppd_name = "HP-Fax2-hplip" # Fixed width (2528 pixels) and 300dpi rendering
-            nick = "HP Fax 2"
-        if self.mq.get('fax-type', FAX_TYPE_NONE) == FAX_TYPE_LEDM:
-            fax_ppd_name = "HP-Fax4-hplip" # Fixed width (1728 pixels) and 200dpi rendering
-            nick = "HP Fax 4"
-        else:
-            fax_ppd_name = "HP-Fax-hplip" # Standard
-            nick = "HP Fax"
-
-        ppds = []
-
-        log.debug("Searching for fax file %s..." % fax_ppd_name)
-
-        ppd_dir = sys_conf.get('dirs', 'ppd')
-        for f in utils.walkFiles(ppd_dir, pattern="HP-Fax*.ppd*", abs_paths=True):
-            ppds.append(f)
-
-        for f in ppds:
-            if f.find(fax_ppd_name) >= 0:
-                fax_ppd = f
-                log.debug("Found PDD file: %s" % fax_ppd)
-                log.debug("Nickname: %s" % cups.getPPDDescription(fax_ppd))
-                break
-        else:
+        back_end, is_hp, bus, model, serial, dev_file, host, zc, port = \
+                device.parseDeviceURI(self.device_uri)
+        norm_model = models.normalizeModelName(model).lower()
+        fax_ppd,fax_ppd_name, nick = cups.getFaxPPDFile(self.mq, norm_model)
+        # Fax ppd not found
+        if not fax_ppd:
             QApplication.restoreOverrideCursor()
             log.error("Fax PPD file not found.")
 
@@ -971,9 +929,8 @@ class SetupForm(SetupForm_base):
 
             self.FailureUI(self.__tr("<b>Fax queue setup failed.</b><p>Please restart CUPS and try again."))
         else:
-            pass
-            # TODO:
-            #service.sendEvent(self.hpssd_sock, EVENT_CUPS_QUEUES_CHANGED, device_uri=self.fax_uri)
+            # sending Event to add this device in hp-systray
+            utils.sendEvent(EVENT_CUPS_QUEUES_CHANGED,self.fax_uri, self.fax_name)
 
         QApplication.restoreOverrideCursor()
 
@@ -1070,6 +1027,13 @@ class PasswordDialog(QDialog):
 
         self.connect(self.okPushButton,SIGNAL("clicked()"),self.accept)
         self.connect(self.passwordLineEdit,SIGNAL("returnPressed()"),self.accept)
+
+    def setDefaultUsername(self, defUser, allowUsernameEdit = True):
+        self.usernameLineEdit.setText(defUser)
+        if not allowUsernameEdit:
+            self.usernameLineEdit.setReadOnly(True)
+            self.usernameLineEdit.setPaletteBackgroundColor(QColor("lightgray"))
+
     def getUsername(self):
         return unicode(self.usernameLineEdit.text())
 
@@ -1087,11 +1051,13 @@ class PasswordDialog(QDialog):
         return qApp.translate("PasswordDialog",s,c)
 
 
-
-def showPasswordUI(prompt):
+def showPasswordUI(prompt, userName=None, allowUsernameEdit=True):
     try:
         dlg = PasswordDialog(prompt, None)
 
+        if userName != None:
+            dlg.setDefaultUsername(userName, allowUsernameEdit)
+
         if dlg.exec_loop() == QDialog.Accepted:
             return (dlg.getUsername(), dlg.getPassword())
 
