@@ -16,7 +16,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Authors: Don Welch, Yashwant Kumar Sahu
+# Authors: Don Welch, Yashwant Kumar Sahu, Sanjay Kumar Sharma
 #
 
 # Std Lib
@@ -356,12 +356,12 @@ class PrintSettingsToolbox(QToolBox):
                     self.job_options['pagerange'] = current
 
                     # page set (COMBO/PICKONE)
-                    current = current_options.get('pageset', '')
+                    current = current_options.get('pageset', 'all')
                     self.addControlRow("pageset", self.__tr("Page Set"),
                         cups.PPD_UI_PICKONE, current,
-                        [('all', self.__tr("All pages")),
-                         ('even', self.__tr("Even pages")),
-                         ('odd', self.__tr("Odd pages"))], 'all', job_option=True)
+                        [('all', self.__tr("AllPages")),
+                         ('even', self.__tr("Even")),
+                         ('odd', self.__tr("Odd"))], 'all', job_option=True)
 
                     self.job_options['pageset'] = current
 #                    if current == u'even':
@@ -934,6 +934,17 @@ class PrintSettingsToolbox(QToolBox):
 
             DefaultButton.setText("Default")
 
+            #type of 'value' and 'default' can be unicode (ppd values), str, int or boolean, so we need to typecast it to bool for easy comparison
+            if value == True or value == 'True' or value == 'true':
+               value = True;
+            else:
+               value = False;
+
+            if default == True or default == 'True' or default == 'true':
+               default = True;
+            else:
+               default = False;
+
             if value == default:
                 DefaultButton.setEnabled(False)
             self.connect(DefaultButton, SIGNAL("clicked()"), self.DefaultButton_clicked)
@@ -1582,6 +1593,10 @@ class PrintSettingsToolbox(QToolBox):
     def BoolRadioButtons_clicked(self, b): # cups.PPD_UI_BOOLEAN
         sender = self.sender()
         b = int(b)
+        if sender.default == True or sender.default == "True" or sender.default == "true":
+            sender.default = int(True)
+        else:
+            sender.default = int(False)
 
         if b == sender.default:
             self.removePrinterOption(sender.option)
@@ -1621,6 +1636,10 @@ class PrintSettingsToolbox(QToolBox):
         sender.setEnabled(False)
 
         if sender.typ == cups.PPD_UI_BOOLEAN: # () On  (*) Off
+            if sender.default == True or sender.default == 'True' or sender.default == 'true': 
+                sender.default = True
+            else:
+                sender.default = False
             if sender.default:
                 sender.control[0].setChecked(True)
                 sender.control[0].setFocus(Qt.OtherFocusReason)
@@ -1704,13 +1723,18 @@ class PrintSettingsToolbox(QToolBox):
 
 
     def PageRangeEdit_editingFinished(self):
-        sender, x = self.sender(), []
-        t, ok = unicode(sender.text()), True
+        sender = self.sender()
+        t, ok, x = self.job_options['pagerange'], True, []
+
+        #[Sanjay]Start Range Validation here as the editing is finished
         try:
-            x = utils.expand_range(t)
+            x = utils.expand_range(t)   
         except ValueError:
             ok = False
 
+        if t == '':
+            ok = False
+
         if ok:
             if 0 in x:
                 ok = False
@@ -1721,12 +1745,7 @@ class PrintSettingsToolbox(QToolBox):
                         ok = False
                         break
 
-        if ok:
-            t = utils.collapse_range(x)
-            sender.setText(QString(t))
-            self.job_options['pagerange'] = t
-
-        else:
+        if not ok:
             self.job_options['pagerange'] = ''
             log.error("Invalid page range: %s" % t)
             FailureUI(self, self.__tr("<b>Invalid page range.</b><p>Please enter a range using page numbers (1-999), dashes, and commas. For example: 1-2,3,5-7</p>"))
@@ -1734,15 +1753,7 @@ class PrintSettingsToolbox(QToolBox):
 
 
     def PageRangeEdit_textChanged(self, t):
-        sender, x, t = self.sender(), [], unicode(t)
-        try:
-            x = utils.expand_range(t)
-        except ValueError:
-            self.job_options['pagerange'] = ''
-            log.error("Invalid page range: %s" % t)
-        else:
-            self.job_options['pagerange'] = t
-
+        self.job_options['pagerange'] = unicode(t) # Do range validation only in PageRangeEdit_editingFinished method
 
     #
     # Job Storage
