@@ -74,6 +74,7 @@ HIDE_INACTIVE_DELAY = 5000
 BLIP_DELAY = 2000
 SET_MENU_DELAY = 1000
 MAX_MENU_EVENTS = 10
+UPGRADE_CHECK_DELAY=24*60*60*1000 		#1 day
 
 ERROR_STATE_TO_ICON = {
     ERROR_STATE_CLEAR:        QSystemTrayIcon.Information,
@@ -205,7 +206,14 @@ class HistoryDevice(QObject):
 class SystraySettingsDialog(QDialog):
     def __init__(self, parent, systray_visible, polling,
                  polling_interval, systray_messages,
-                 device_list=None):
+                 device_list=None,
+                 upgrade_notify=True,
+                 upgrade_pending_time=0,
+                 upgrade_last_update_time=0,
+                 upgrade_msg=""
+                 ):
+#                 upgrade_pending_update_time=0,
+
 
         QDialog.__init__(self, parent)
 
@@ -219,6 +227,10 @@ class SystraySettingsDialog(QDialog):
 
         self.polling = polling
         self.polling_interval = polling_interval
+        self.upgrade_notify =upgrade_notify
+        self.upgrade_last_update_time=upgrade_last_update_time
+        self.upgrade_pending_time=upgrade_pending_time
+        self.upgrade_msg=upgrade_msg
 
         self.initUi()
         self.SystemTraySettings.updateUi()
@@ -235,7 +247,10 @@ class SystraySettingsDialog(QDialog):
         self.SystemTraySettings.initUi(self.systray_visible,
                                        self.polling, self.polling_interval,
                                        self.device_list,
-                                       self.systray_messages)
+                                       self.systray_messages,
+                                       self.upgrade_notify,
+                                       self.upgrade_pending_time,
+                                       self.upgrade_msg)
 
         sizePolicy = QSizePolicy(QSizePolicy.Expanding,QSizePolicy.Expanding)
         sizePolicy.setHorizontalStretch(0)
@@ -260,6 +275,9 @@ class SystraySettingsDialog(QDialog):
         #QMetaObject.connectSlotsByName(self)
 
         self.setWindowTitle(self.__tr("HP Device Manager - System Tray Settings"))
+        self.setWindowIcon(QIcon(load_pixmap('hp_logo', '128x128')))
+#        pm = load_pixmap("hp_logo", "32x32")
+#        self.prop_icon = QIcon(pm)
 
 
     def acceptClicked(self):
@@ -268,6 +286,7 @@ class SystraySettingsDialog(QDialog):
         self.polling_interval = self.SystemTraySettings.polling_interval
         self.device_list = self.SystemTraySettings.device_list
         self.systray_messages = self.SystemTraySettings.systray_messages
+        self.upgrade_notify =self.SystemTraySettings.upgrade_notify
         self.accept()
 
 
@@ -276,13 +295,14 @@ class SystraySettingsDialog(QDialog):
 
 
 
+
 class SystemTrayApp(QApplication):
     def __init__(self, args, read_pipe):
         QApplication.__init__(self, args)
 
         self.menu = None
         self.read_pipe = read_pipe
-        self.fmt = "64s64sI32sI64sf"
+        self.fmt = "80s80sI32sI80sf"
         self.fmt_size = struct.calcsize(self.fmt)
         self.timer_active = False
         self.active_icon = False
@@ -325,8 +345,14 @@ class SystemTrayApp(QApplication):
         self.tray_icon.setIcon(self.prop_active_icon)
         self.active_icon = True
 
+        self.handle_hplip_updation()
         QTimer.singleShot(SET_MENU_DELAY, self.initDone)
 
+        self.timer = QTimer()
+        self.timer.connect(self.timer,SIGNAL("timeout()"),self.handle_hplip_updation)
+        self.timer.start(UPGRADE_CHECK_DELAY)
+
+
 
     def initDone(self):
         self.tray_icon.setIcon(self.prop_icon)
@@ -334,6 +360,8 @@ class SystemTrayApp(QApplication):
 
         self.setMenu()
 
+    def resetDevice(self):
+        devices.clear()
 
     def addDevice(self, device_uri):
         try:
@@ -344,6 +372,46 @@ class SystemTrayApp(QApplication):
             devices[device_uri].needs_update = True
 
 
+    def handle_hplip_updation(self):
+        log.debug("handle_hplip_updation upgrade_notify =%d"%(self.user_settings.upgrade_notify))
+        path = utils.which('hp-upgrade')
+        if self.user_settings.upgrade_notify is False:
+            log.debug("upgrade notification is disabled in systray ")
+            if path:
+                path = os.path.join(path, 'hp-upgrade')
+                log.debug("Running hp-upgrade: %s " % (path))
+                # this just updates the available version in conf file. But won't notify
+                os.spawnlp(os.P_NOWAIT, path, 'hp-upgrade', '--check')
+                time.sleep(5)
+                try:
+                    os.waitpid(0, os.WNOHANG)
+                except OSError:
+                    pass
+            return
+            
+            
+        current_time = time.time()
+    
+        if int(current_time) > self.user_settings.upgrade_pending_update_time:
+            path = utils.which('hp-upgrade')
+            if path:
+                path = os.path.join(path, 'hp-upgrade')
+                log.debug("Running hp-upgrade: %s " % (path))
+                os.spawnlp(os.P_NOWAIT, path, 'hp-upgrade', '--notify')
+                time.sleep(5)
+            else:
+                log.error("Unable to find hp-upgrade --notify on PATH.")
+        else:
+            log.debug("upgrade schedule time is not yet completed. schedule time =%d current time =%d " %(self.user_settings.upgrade_pending_update_time, current_time))
+        
+        try:
+            os.waitpid(0, os.WNOHANG)
+        except OSError:
+            pass
+
+
+
+
 
     def setMenu(self):
         self.menu = QMenu()
@@ -417,16 +485,41 @@ class SystemTrayApp(QApplication):
             return
 
         self.sendMessage('', '', EVENT_DEVICE_STOP_POLLING)
+#        sys_conf
+        cur_vers = sys_conf.get('hplip', 'version')
+        self.user_settings.load()
+        installed_time =time.strftime("%d-%m-%Y", time.localtime(self.user_settings.upgrade_last_update_time))
+        if utils.Is_HPLIP_older_version(cur_vers, self.user_settings.latest_available_version):
+            if int(time.time()) < self.user_settings.upgrade_pending_update_time :
+                postponed_time =time.strftime("%d-%m-%Y", time.localtime(self.user_settings.upgrade_pending_update_time))
+                upgrade_msg ="HPLIP-%s version was installed on %s.\n\nNew version of HPLIP-%s is available for upgrade. HPLIP upgrade is scheduled on %s." %(cur_vers,installed_time , self.user_settings.latest_available_version, postponed_time)
+            elif self.user_settings.upgrade_last_update_time:
+                upgrade_msg ="HPLIP-%s version was installed on %s.\n\nNew version of HPLIP-%s is available for upgrade." %(cur_vers,installed_time , self.user_settings.latest_available_version)
+            else:
+                upgrade_msg ="HPLIP-%s version was installed.\n\nNew version of HPLIP-%s is available for upgrade." %(cur_vers, self.user_settings.latest_available_version)
+        elif self.user_settings.upgrade_last_update_time:
+            upgrade_msg ="HPLIP-%s version was installed on %s."%(cur_vers, installed_time)
+        else: 
+            upgrade_msg ="HPLIP-%s version was installed."%(cur_vers)
+            
+        
         try:
             dlg = SystraySettingsDialog(self.menu, self.user_settings.systray_visible,
                                         self.user_settings.polling, self.user_settings.polling_interval,
                                         self.user_settings.systray_messages,
-                                        self.user_settings.polling_device_list)
+                                        self.user_settings.polling_device_list,
+                                        self.user_settings.upgrade_notify,
+                                        self.user_settings.upgrade_pending_update_time,
+                                        self.user_settings.upgrade_last_update_time,
+                                        upgrade_msg)
+
 
             if dlg.exec_() == QDialog.Accepted:
                 self.user_settings.systray_visible = dlg.systray_visible
                 self.user_settings.systray_messages = dlg.systray_messages
-
+                self.user_settings.upgrade_notify = dlg.upgrade_notify
+        
+                log.debug("HPLIP update  notification = %d"%(self.user_settings.upgrade_notify))
                 self.user_settings.save()
 
                 if self.user_settings.systray_visible == SYSTRAY_VISIBLE_SHOW_ALWAYS:
@@ -489,7 +582,7 @@ class SystemTrayApp(QApplication):
         log.debug("Exiting")
         self.sendMessage('', '', EVENT_SYSTEMTRAY_EXIT)
         self.quit()
-
+        del self.tray_icon
 
     def toolboxTriggered(self):
         try:
@@ -547,6 +640,13 @@ class SystemTrayApp(QApplication):
                     event = device.Event(*struct.unpack(self.fmt, m[:self.fmt_size]))
 
                     m = m[self.fmt_size:]
+                    
+                    if event.event_code == EVENT_CUPS_QUEUES_CHANGED:
+                        self.resetDevice()
+                        for d in device.getSupportedCUPSDevices(back_end_filter=['hp', 'hpfax']):
+                            self.addDevice(d)
+                            
+                        self.setMenu()
 
                     if event.event_code == EVENT_USER_CONFIGURATION_CHANGED:
                         log.debug("Re-reading configuration (EVENT_USER_CONFIGURATION_CHANGED)")
@@ -706,7 +806,7 @@ def run(read_pipe):
     app.setQuitOnLastWindowClosed(False) # If not set, settings dlg closes app
 
     i = 0
-    while i < 10:
+    while i < 60:
         if QSystemTrayIcon.isSystemTrayAvailable():
             break
         time.sleep(1.0)
